import java.awt.*;
import java.awt.event.*;

import sv.kernel.*;
import sv.ui.*;

/** 
 * GCanvas3D.java
 * Define a canvas to draw the graph of 3d data
 * @version 1.0    4/1/98
 * @author Byeongseob Ki
 */
 
public class GCanvas3D extends GCanvas 
                implements KeyListener, MouseMotionListener, WheelClient {

   private GFrame3D gframe;
   svserver svServer;

   //////////////////////////////////////
   // POSITION RELATED
   private int width = 320, height = 320;
	
   public  float xfac = 0.f;
   public float scalefudge = 1.f;
   int click_x = 0, click_y = 0;

   private float xtheta = 0.f;
   private float ytheta = 0.f;

   //////////////////////////////////////
   // double buffering stuff 
   private Dimension offScreenSize;


   //////////////////////////////////////
   // Datas set and time data
   TriObject isotri;

   private float xmax, ymax, zmax, xmin, ymin, zmin;

   private boolean ctrlDragged = false; 

  /**
   * The constructor of GCanvas3D
   */
   public GCanvas3D(GFrame3D gframe) {
     this.gframe = gframe;
     this.svServer = gframe.svServer;

     addKeyListener(this);
     addMouseMotionListener(this);
     enableEvents(AWTEvent.MOUSE_EVENT_MASK);  
   }

   /**
    * return offscreen image
    */
   public Image getImage() {
     return offScrImage;
   }

   public void decideSize() {

      float xw, yw, zw;
       
       isotri = gframe.isoObjects[gframe.currentTime];

       // calculate the size of the object etc. 
       isotri.findBB();
       xw = isotri.xmax - isotri.xmin;
       yw = isotri.ymax - isotri.ymin;
       zw = isotri.zmax - isotri.zmin;

       xmax = isotri.xmax;
       ymax = isotri.ymax;
       zmax = isotri.zmax;
       xmin = isotri.xmin;
       ymin = isotri.ymin;
       zmin = isotri.zmin;

       float xgap = (float) ((xmax - xmin)/10.0f);
       float ygap = (float) ((ymax - ymin)/10.0f);


       if (yw > xw)
	   xw = yw;
       if (zw > xw)
	   xw = zw;

       float f1 = getSize().width / xw;
       float f2 = getSize().height / xw;
       xfac = 0.7f * (f1 < f2 ? f1 : f2) * scalefudge;

       gframe.xcam0 = (int)((xmax - xmin + 1.f)/2);
       gframe.ycam0 = (int)((ymax - ymin + 1.f)/2);
       gframe.zcam0 = (int)((zmax - zmin + 1.f)*2); 

       isotri.xcam = gframe.xcam0;
       isotri.ycam = gframe.ycam0;
       isotri.zcam = gframe.zcam0;

       gframe.prevxcam = gframe.xcam0;
       gframe.prevycam = gframe.ycam0;
       gframe.prevzcam = gframe.zcam0;

   }

  /**
   * set position
   * @param xtheta x rotation value
   * @param ytheta y rotation value
   */
  public void setPosition(float xtrans, float ytrans) {

    gframe.config.xtheta = xtrans;
    gframe.config.ytheta = ytrans;
    xtheta = xtrans;
    ytheta = ytrans;
    rotate(xtrans, ytrans);
  }


   public void paint (Graphics g) {
      paintDone = false;

      Dimension d = getSize();

      if ((width != getSize().width) || (height != getSize().height)) {
         width = getSize().width;
         height = getSize().height;
         offScrImage = null;
      }
		
      if (offScrImage == null) {
          offScrImage = createImage(width, height);
          offScrGc = offScrImage.getGraphics();
      }

      decideSize();

      /////////////////////////////////////////////////////////////
      // printing
      printing = g instanceof PrintGraphics;
      if (printing) {

        // modifies variables
        Graphics savedgc = offScrGc;
        offScrGc = g;

        Dimension pagedimension = 
               ((PrintGraphics)g).getPrintJob().getPageDimension();
      
        printwidth = pagedimension.width;
        printheight = height * width / width;

        if (printheight > pagedimension.height) {
          printheight = pagedimension.height;
          printwidth = width * height / height;
        }

        // erase whole canvas
        offScrGc.setColor(getBackground());
        offScrGc.fillRect(0, 0, printwidth, printheight);

        isotri.resetPosition();
        isotri.translate(-(xmin + xmax) / 2,
		       -(ymin + ymax) / 2,  -(zmin + zmax) / 2);
        isotri.relativePosition(gframe.amat);

        isotri.scale(xfac, -xfac, 16 * xfac / width);
        isotri.translate(width / 2, height / 2, 8);
        isotri.transformed = false;
        isotri.paint(offScrGc);

        offScrGc = savedgc;
        paintDone = true;
        return;
      }

      // erase all first
      offScrGc.setColor(getBackground());
      offScrGc.fill3DRect(0, 0, width, height, true);

      isotri.resetPosition();
      isotri.translate(-(xmin + xmax) / 2,
		       -(ymin + ymax) / 2,  -(zmin + zmax) / 2);
      isotri.relativePosition(gframe.amat);

      //float saved = 21.855555f;
      //System.out.println( (64 * xfac / width));
      isotri.scale(xfac, -xfac, 16 * xfac / width);
      //      isotri.scale(saved, -saved, 64 * xfac / width);
      isotri.translate(width / 2, height / 2, 8);
      isotri.transformed = false;
      isotri.paint(offScrGc);

      // draw time string
      FontMetrics fm = offScrGc.getFontMetrics();
      offScrGc.setColor(Color.black);
      offScrGc.drawString("Time: " + gframe.timeArr[gframe.currentTime],
                  5, fm.getAscent()+10);

      DataSet3D isoData = (DataSet3D)gframe.dataSetVector.firstElement();
      
      try {
      gframe.monitor.display((gframe.currentTime+1) + " / " + gframe.noOfTime +
        " threshold: " + isoData.timeData[gframe.currentTime].getThreshold() +
        " (" + isotri.getNTRI() + " triangles)", false);
      } catch (Exception e) {
      gframe.monitor.display((gframe.currentTime+1) + " / " + gframe.noOfTime +
        " (" + isotri.getNTRI() + " triangles)", false);
      }

      g.drawImage(offScrImage, 0, 0, this);
      paintDone = true;
   }



  // overrided function
  public void processMouseEvent(MouseEvent e) {
     int x = e.getX();
     int y = e.getY();

     if (e.getID() == MouseEvent.MOUSE_PRESSED) { 
        gframe.prevx = x;
	gframe.prevy = y;
        click_x = x;
        click_y = y;

     } else if (e.getID() == MouseEvent.MOUSE_RELEASED) { 
        if (isDragged && ctrlDragged) {
           if (gframe.xgridMI.getState() ||  gframe.ygridMI.getState() ||
                             	       gframe.zgridMI.getState()) {

               int delta = 0;

               if ((gframe.prevy - y) > 0) 
                   delta = (int)((gframe.prevy - y)/80 + 1);
               else if ((gframe.prevy - y) < 0) 
                   delta = (int)((gframe.prevy - y)/80 - 1);
               else if ((gframe.prevy - y) == 0) return;

              gframe.updateGridInfo(delta);
	   }
        } else {
	  /*
           String[] args = { String.valueOf(xtheta), 
                             String.valueOf(ytheta) };

           svServer.fmanager.action3d(gframe, 
                          CommandSet3D.POSITIONCHANGE, args, true);
	  */
        }

        isDragged = false;
        ctrlDragged = false;
        repaint();
     }
  }

  /**
   * Respond to mouse drag by rotating the object,
   * and updating the position of the camera. 
   */
  public void mouseDragged(MouseEvent e) {

     int x = e.getX();
     int y = e.getY();

     isDragged = true;

     if (e.isControlDown()) {
        ctrlDragged = true;

     } else {
        // Rotate object.  A mouse movement right across display would
        //  result in a 360 turn of the object.

        xtheta = (gframe.prevy - y) * 360.0f / width;
        ytheta = (x - gframe.prevx) * 360.0f / height;

        gframe.prevx = x;
        gframe.prevy = y;

        rotate(xtheta, ytheta);
     }
  }

   public void mouseMoved(MouseEvent e) {
      // nothing to do
   }

  public void keyTyped(KeyEvent e) {}
  public void keyReleased(KeyEvent e) {}
  public void keyPressed(KeyEvent e) {
      if (e.isControlDown()) {   
        switch(e.getKeyCode()) {
           case 65:                       // ctrl-a animation
              svServer.fmanager.action3d(gframe, CommandSet3D.ANIMATE, 
                                                              null, true);
              break;

           case 83:                       // ctrl-s stop
              String[] arg = new String[1];
              arg[0] = String.valueOf(gframe.currentTime);
              svServer.fmanager.action3d(gframe, CommandSet3D.STOP, 
                                                              arg, true);
              break;

           case 78:                       // ctrl-n next
              svServer.fmanager.action3d(gframe, CommandSet3D.NEXT, 
                                                              null, true);
              break;

           case 80:                      // ctrl-p previous
              svServer.fmanager.action3d(gframe, CommandSet3D.PREVIOUS, 
                                                              null, true);
              break;

           case 81:                      // ctrl-q kill window
              svServer.fmanager.action3d(gframe, CommandSet3D.QUIT, 
                                                              null, true);
              break;

           case 71:                      // ctrl-g go specific time
              gframe.doWork(CommandSet3D.GOTO, null);
              break;

        }
      }
  }


   public void rotate(float thetax, float thetay) {
      
      // reset to unit matrix.
      gframe.tmat.unit();

      gframe.tmat.xrot(thetax);
      gframe.tmat.yrot(thetay);
      gframe.amat.mult(gframe.tmat);
	
      // UPDATE CAMERA
	
      // Calculate current distance of camera from origin
      float rad = (float) Math.sqrt(isotri.xcam*isotri.xcam 
         		+ isotri.ycam*isotri.ycam 
		        + isotri.zcam*isotri.zcam);

      // Convert angles into radians (???)
      thetax = (thetax / 360.0f) * 2.0f * 3.145f;	
      thetay = (thetay / 360.0f) * 2.0f * 3.145f;	

      // Calculate the new camera position based on old position & angle
      isotri.xcam = (float) (gframe.prevxcam * Math.cos(thetay) 
			- gframe.prevzcam * Math.sin(thetay));
      isotri.zcam = (float) (gframe.prevxcam * Math.sin(thetay) 
			+ gframe.prevzcam * Math.cos(thetay));

      gframe.prevxcam = isotri.xcam;
      gframe.prevzcam = isotri.zcam;
	
      // ??? Why is zcam calculated twice???
      isotri.ycam = (float) (gframe.prevycam * Math.cos(thetax) - gframe.prevzcam * Math.sin(thetax));
      isotri.zcam = (float) (gframe.prevycam * Math.sin(thetax) + gframe.prevzcam * Math.cos(thetax));
	
      gframe.prevzcam = isotri.zcam;
      gframe.prevycam = isotri.ycam;

      repaint();
   }

   public void update (Graphics g) {
      paint(g);
   }

   public void onWheelMotion(Wheel wheel, int delta) {
 
      float xtheta = this.xtheta;
      float ytheta = this.ytheta;

      // ????? 
      if (wheel.getOrientation () == Wheel.HORIZONTAL) {
          ytheta = delta * 360.0f / height;        
          gframe.prevy = gframe.prevy - delta;
      } else {
          xtheta = delta * 360.0f / width;         
          gframe.prevx = delta - gframe.prevx;
      }

      rotate(xtheta, ytheta);
   }

   public void onWheelReset(Wheel wheel) {
   }

} // end of class


