import java.awt.*;
import java.awt.event.*;
import java.io.*;

import com.sun.java.swing.*;
import sv.wizard.*;

/** 
 * FilterIDE.java
 * Filter editing IDE
 * @version 1.0   4/5/98 
 * @author Byeongseob Ki
 */

class FilterIDE implements IDEOwnerInterface {

   svserver svServer;
 
   JFrame frame;
   IDEPanel ide;

   String COMPILER = "javac";   
   String filterhome = null;
   FilterCompile compile = null;
   FilterBuild build = null;
   FilterDelete deleter = null;

   public FilterIDE(svserver svServer, String filterhome) {
      this.svServer = svServer;
      this.filterhome = filterhome;

      try {
          frame = new JFrame();
          ide = new IDEPanel();
          ide.setOwner(this);

          frame.setTitle("Filter Editor");
	  frame.setBackground(Color.lightGray);
  	  frame.getContentPane().setLayout(new BorderLayout());
	  frame.getContentPane().add("Center", ide);
	  frame.addWindowListener(new AppCloser());
	  frame.pack();
	  frame.setSize(600, 480);
          frame.show();
       } catch (Throwable t) {
          System.out.println("uncaught exception: " + t);
          t.printStackTrace();
      }
   }

  
   public void ownerAction(ActionEvent e) {
      String command = e.getActionCommand();

      if(command.equals("exit")) {
         ide = null;
         frame.dispose();
         if (build != null) {
            build.dispose();
            build = null;
         }
         svServer.mainb.editBtn.setEnabled(true);

      } else if (command.equals("wizard")) {
         FilterEditor pad = new FilterEditor(ide, filterhome);
	 pad.pack();
	 pad.show();

      /////////////////////////////////////////////////////////////
      // COMPILE
      } else if (command.equals("compile")) {
         String direc = ide.getCurrDirectory();
         String cfile = ide.getCurrFile();
         String filename = direc + cfile;
         int idx = filename.lastIndexOf(".java");

         if (direc == null || cfile == null) {
            ide.message("No opened file.");
            return;
         } else if (idx < 0) {
            ide.message("Opend file is not Java file.");
         }

         ide.message("Compiling: " + cfile);
         if (compile != null) 
            compile.dispose();

         compile = new FilterCompile(this, filename, COMPILER);

         if (compile.error) {
            compile.setSize(500, 200);
            compile.setVisible(true);
         } else {
            compile.dispose();
            ide.message("Compiled successfully.");            
         }

      /////////////////////////////////////////////////////////////
      // BUILD
      } else if (command.equals("build")) {
         String direc = ide.getCurrDirectory();
         String cfile = ide.getCurrFile();
         String filename = direc + cfile;
         int idx = filename.lastIndexOf(".java");
         String packagename = "";
         int dim = 1;

         if (direc == null || cfile == null) {
            ide.message("No opened file.");
            return;
         } else if (idx < 0) {
            ide.message("Opend file is not Java file.");
         }

         String classname = filename.substring(0, idx);
         ide.message("Building " + classname);
         ide.progressSetValue(0);
         ide.progressSetMinimum(0);
         ide.progressSetMaximum(5);

         File newclass = new File(classname + ".class");
         if (!newclass.exists()) {
            ide.message("Can't find " + classname + ".class file.");
            ide.progressSetValue(0);
            return;
         }
         idx = cfile.lastIndexOf(".java");
         classname = cfile.substring(0, idx);
	 //System.out.println(classname);

         ide.message("Parsing source program.");
    
         try {
            BufferedReader reader = new BufferedReader(new FileReader(filename));
            boolean done1 = false, done2 = false;
            String str;

            while ( (str = reader.readLine()) != null) {
               str =  str.trim();
               if (str.startsWith("//"))
                  continue;
               
               if (str.startsWith("package")) {
                  idx = str.indexOf(";");
                  packagename = str.substring(8, idx).trim();
		  //System.out.println(packagename);
                  if (done2) break;
                  done1 = true;

               } else if (str.startsWith("super(")) {
                  idx = str.indexOf(");");
                  str = str.substring(6, idx).trim();
                  if (str.indexOf("DataSet.ONED") >= 0)
                     dim = 1;
                  else if (str.indexOf("DataSet.TWOD") >= 0) 
                     dim = 2;
                  else {
                     ide.message("Can't know the rank of the target data.");
                     ide.progressSetValue(0);
                     return;
                  }
		  //System.out.println(dim);
                  
                  if (done1) break;
                  done2 = true;
               }
            }
            reader.close();

         } catch (Exception Ex) {
            ide.message("Parsing source program error.");
            ide.progressSetValue(0);
            return;
         }
         
         ide.progressSetValue(1);         

         if (build != null) 
            build.dispose();

         build = new FilterBuild(this, dim, classname, packagename);
	 build.setSize(400, 450);
         build.setVisible(true);

      } else if (command.equals("delete")) {
         if (deleter != null) 
            return;

         deleter = new FilterDelete(this);
	 deleter.setSize(400, 420);
         deleter.setVisible(true);
      }
   }

   /**
    * To shutdown when run as an application.  This is a
    * fairly lame implementation.   A more self-respecting
    * implementation would at least check to see if a save
    * was needed.
    */
   protected static final class AppCloser extends WindowAdapter {
       public void windowClosing(WindowEvent e) {

	}
   }


} // end of class

