package sv.kernel;

import java.awt.*;
import java.io.*;
import java.util.*;

import sv.util.*;

/** 
 * TimeData3D.java
 * Define point data in each time
 * @version 1.0    10/1/97
 * @author Byeongseob Ki
 */

public class TimeData3D extends TimeData  {

  public float[][] vertex;
  public float[][] normal;
  public int[][] triangles;
  public float[][] boundingbox;
  public GridInfo3D surfaceGrids;

  int nvert, ntri;
  int dimx, dimy, dimz;
  float zmin, zmax;
  float threshold;

  public boolean BOUNDBOX_POSSIBLE = false;
  public boolean GRID_AVAILABLE = false;

  /**
   * constructor for 3d dataset
   */
  public TimeData3D(float time, int nvert, float[][] vertex, float[][] normal, 
                 int ntri, int[][] triangles, int dx, int dy, int dz, 
                 float threshold, float min, float max) {

     this.time = time;
     this.nvert = nvert;
     this.vertex = vertex;
     this.normal = normal;
     this.ntri = ntri;
     this.triangles = triangles;
     this.threshold = threshold;

     dimx = dx;
     dimy = dy;
     dimz = dz;

     BOUNDBOX_POSSIBLE = true;
     boundingbox = new float[8][3];
     boundingbox[0][0] = 0.0f;
     boundingbox[0][1] = 0.0f;
     boundingbox[0][2] = 0.0f;
     boundingbox[1][0] = (float)dimx;
     boundingbox[1][1] = 0.0f;
     boundingbox[1][2] = 0.0f;
     boundingbox[2][0] = (float)dimx;
     boundingbox[2][1] = (float)dimy;
     boundingbox[2][2] = 0.0f;
     boundingbox[3][0] = 0.0f;
     boundingbox[3][1] = (float)dimy;
     boundingbox[3][2] = 0.0f;
     boundingbox[4][0] = 0.0f;
     boundingbox[4][1] = 0.0f;
     boundingbox[4][2] = (float)dimz;
     boundingbox[5][0] = (float)dimx;
     boundingbox[5][1] = 0.0f;
     boundingbox[5][2] = (float)dimz;
     boundingbox[6][0] = (float)dimx;
     boundingbox[6][1] = (float)dimy;
     boundingbox[6][2] = (float)dimz;
     boundingbox[7][0] = 0.0f;
     boundingbox[7][1] = (float)dimy;
     boundingbox[7][2] = (float)dimz;
   
     zmin = min;
     zmax = max;
  }

  /**
   * constructor for 3d dataset
   */
  public TimeData3D(float time, int nvert, 
                    float[] xvert, float[] yvert, float[] zvert, float[] cvals,
                    int nc, int[] cl) {

    //////////////////////////
    // NORMAL IS NULL ....
    //////////////////////////

// color values are ignored ...

     this.time = time;
     this.nvert = nvert;

     float xmin = Float.MAX_VALUE;
     float xmax = Float.MIN_VALUE;
     float ymin = Float.MAX_VALUE;
     float ymax = Float.MIN_VALUE;
     zmin = Float.MAX_VALUE;
     zmax = Float.MIN_VALUE;

     vertex = new float[nvert][3];
     for (int i=0; i<nvert; i++) {
        vertex[i][0] = xvert[i];
        vertex[i][1] = yvert[i];
        vertex[i][2] = zvert[i];
        
        if (xvert[i] < xmin) xmin = xvert[i]; 
        else if (xvert[i] > xmax) xmax = xvert[i];

	if (yvert[i] < ymin) ymin = yvert[i]; 
        else if (yvert[i] > ymax) ymax = yvert[i];

	if (zvert[i] < zmin) zmin = zvert[i]; 
        else if (zvert[i] > zmax) zmax = zvert[i];

     }

     dimx = (int)(xmax-xmin+1);
     dimy = (int)(ymax-ymin+1);
     dimz = (int)(zmax-zmin+1);

//System.out.println("x range = ["+xmin+", "+xmax+"]");
//System.out.println("y range = ["+ymin+", "+ymax+"]");
//System.out.println("z range = ["+zmin+", "+zmax+"]");

     ntri = 0;     

     int tris[][] = new int[nc/2][3];
   
     Vector v;
     int v1=0, v2=0, v3=0, index=0;

     for (int i=0; i<nc; i++) {

        if (cl[i] == -1) {
          index = 0;
        } else {
//System.out.println(cl[i] + " " + index);
          if (index == 0) {
            v1 = cl[i]; 
            index = 1;
          } else if (index == 1) {
            v2 = cl[i]; 
            index = 2;
          } else if (index == 2) {
            v3 = cl[i]; 
            tris[ntri][0] = v1;
            tris[ntri][1] = v2;
            tris[ntri][2] = v3;
//System.out.println(v1 + " " + v2 + " " + v3);
            ntri++;
            v2 = v3;
          }
        }
     }
 
     triangles = new int[ntri][3];
     for (int i=0; i<ntri; i++) {
       triangles[i][0] = tris[i][0];
       triangles[i][1] = tris[i][1];
       triangles[i][2] = tris[i][2];
     }

     this.threshold = 0.0f;
//System.out.println(ntri + " triangles are created.");
  }

  /**
   * set surface grids
   */
  public void setSurfaceGrid(GridInfo3D grids) {
     if (grids == null) {
        System.out.println("Error: Surface grid data is null ...");
        return;
     }
     surfaceGrids = grids;
     GRID_AVAILABLE = true;
  }

  /** 
   *  sending data by using DataOutputStream
   */
  public void sendingData(DataOutputStream out) {

    try {
       // send time
       out.writeFloat(time);

       // send dimensions
       out.writeInt(dimx);
       out.writeInt(dimy);
       out.writeInt(dimz);

       // send threshold
       out.writeFloat(threshold);

       // send zmin and zmax
       out.writeFloat(zmin);
       out.writeFloat(zmax); 

       // send reserved for future use (2 items)
       out.writeInt(0);
       out.writeInt(0);

       // send vertex
       out.writeInt(nvert);
       for (int i=0; i<nvert; i++) {
           out.writeFloat(vertex[i][0]);
           out.writeFloat(vertex[i][1]);
           out.writeFloat(vertex[i][2]);
           out.writeFloat(normal[i][0]);
           out.writeFloat(normal[i][1]);
           out.writeFloat(normal[i][2]);
       }

       // send triangle info
       out.writeInt(ntri);
       for (int i=0; i<ntri; i++) {
           out.writeInt(triangles[i][0]);
           out.writeInt(triangles[i][1]);
           out.writeInt(triangles[i][2]);
       }

    } catch (Exception e) {
       System.out.println("Data sending error(TimeData2D).");
    } 
  }


  public int getDimX() {
     return dimx;
  }	

  public int getDimY() {
     return dimy;
  }	

  public int getDimZ() {
     return dimz;
  }	

  public float getzmin() {
     return zmin;
  }

  public float getzmax() {
     return zmax;
  }

  public float getThreshold() {
     return threshold;
  }

  public int getNVERT() {
     return nvert;
  }

  public int getNTRI() {
     return ntri;
  }

} // end of class


