import java.awt. *;
import java.io. *;
import java.awt.event. *;
import java.util.*;

import javax.swing.*;
import javax.swing.*;
import javax.swing.*;
import javax.swing.*;

import sv.kernel.*;
import sv.util.*;

public class FilterBuild extends JFrame implements ActionListener {

   private FilterIDE ide;  
   svserver svServer;
   TreePanel trees;  
   JTextField menuField, filterField;

   static final int ONED = 1;
   static final int TWOD = 2;
   static final int THREED = 3;
   static final int VECTOR = 4;

   int currDimension = ONED;
   String classname, packname;

   // constructor
   FilterBuild(FilterIDE parent, int dim, String classname, String packname) {
      super("Filter Builder");
      ide = parent;
      svServer = parent.svServer;
      this.classname = classname;
      this.packname = packname;

      Container container = getContentPane();

      container.setLayout(new BoxLayout(container, BoxLayout.Y_AXIS));
      container.add(trees = new TreePanel());

      JPanel p = new JPanel();
      p.setLayout(new GridLayout(2, 1));
      p.setBorder(new EmptyBorder(5,5,5,5));

      JPanel p2 = new JPanel();
      p2.setLayout(new BoxLayout(p2, BoxLayout.X_AXIS));
      p2.add(new JLabel("Menu Structure: "));
      p2.add(menuField = new JTextField());
      p.add(p2);

      p2 = new JPanel();
      p2.setLayout(new BoxLayout(p2, BoxLayout.X_AXIS));
      p2.add(new JLabel("Menu Item Name: "));
      p2.add(filterField = new JTextField());
      p.add(p2);

      container.add(p);
      container.add(Box.createRigidArea(new Dimension(1, 5)));

      p = new JPanel();
      p.setLayout(new FlowLayout(FlowLayout.RIGHT));
      p.setBorder(new EmptyBorder(5,5,5,5));
      Button btn;
      p.add(btn = new Button("Build"));
      btn.addActionListener(this);
      p.add(btn = new Button("Cancel"));
      btn.addActionListener(this);
      container.add(p);
      validate();
      pack();
   }


   /**
    * overrided method
    */
   public void actionPerformed(ActionEvent evt) {
      String command = evt.getActionCommand();

      if (command.equals("Build")) { 
         String str = menuField.getText();
         if (str != null) str = str.trim();
         Vector pathVec = new Vector();

         if (str != null && str.length() > 1) {
            StringTokenizer st = new StringTokenizer(str, "/");
            while (st.hasMoreTokens()) 
               pathVec.addElement(st.nextToken());
   
            str = (String)pathVec.firstElement();
            if (str.indexOf("Filter(1d)") >= 0 || 
                     str.indexOf("Filter(2d)") >= 0 || 
                     str.indexOf("Filter(3d)") >= 0 || 
                     str.indexOf("Filter(Vector)") >= 0) 
               pathVec.removeElementAt(0);
	 }

         str = filterField.getText();
         if (str == null || str.length() < 1) 
            str = "untitled";
         else 
            str = str.trim();
         pathVec.addElement(str);

         int depth = pathVec.size();
         if (depth > 3) {
  	   ide.ide.message("This filter structure is too deep.");
           ide.ide.progressSetValue(0); 
           return;
         }

         ide.ide.message("Updating sv_menu file");
         ide.ide.progressSetValue(3); 

         int errcode = svServer.menuHandler.insertItem(currDimension, classname, packname, pathVec);

         
         if (errcode == -1) {
            ide.ide.message("Unsupported dimension error.");
            ide.ide.progressSetValue(0); 
            return;
         } else if (errcode == -2) {
            ide.ide.message("Duplicated item error.");
            ide.ide.progressSetValue(0); 
            return;
         } else if (errcode == -3) {
            ide.ide.message("Missing parent node error.");
            ide.ide.progressSetValue(0); 
            return;
         }

         ide.ide.message("Updating filter menu of opened window.");
         ide.ide.progressSetValue(4); 

         Vector frameVec;
         switch (currDimension) {       
  	     case ONED:
                frameVec = svServer.fmanager.get1dFrames();
                
                for (int i=0; i<frameVec.size(); i++) {
                    GFPOneD gframe1d = (GFPOneD)frameVec.elementAt(i);
                    if (gframe1d.getDataType() == DSPlane.NORMAL) {
                       ((GFPOneD)gframe1d).updateFilterMenu();
                    }
                }
                break;

             case TWOD:
                frameVec = svServer.fmanager.get2dFrames();

                for (int i=0; i<frameVec.size(); i++) {
                    GFrame2D gframe2d = (GFrame2D)frameVec.elementAt(i);
                    gframe2d.updateFilterMenu();
                }
                break;

             default:
                ide.ide.message("Unsupported dimension error.");
                ide.ide.progressSetValue(0); 
                return;
          }

         ide.ide.message("Builing completed.");
         ide.ide.progressSetValue(0); 

      } else if (command.equals("Cancel")) { 
         ide.ide.message("Build canceled.");
         ide.ide.progressSetValue(0);
      }

      this.dispose();
   }

   class TreePanel extends JPanel {

       TreePanel() {
          
          String topstr = null;
          Vector level0Vec = new Vector();
          Vector level1Vec = new Vector();
          Vector level2Vec = new Vector();
          Vector level3Vec = new Vector();

          switch (currDimension) {       
  	     case ONED:
                for (Enumeration e = svServer.menuHandler.get1dElements(); e.hasMoreElements();) {
                   MenuInfo info = (MenuInfo)e.nextElement();

                   if (info.level == 0) 
	             level0Vec.addElement(info);
                   else if (info.level == 1) 
	             level1Vec.addElement(info);
                   else if (info.level == 2) 
        	     level2Vec.addElement(info);
                   else if (info.level == 3) 
                     level3Vec.addElement(info);
                }  
                topstr = "Filter(1d)";
                break;

             case TWOD:
                for (Enumeration e = svServer.menuHandler.get2dElements(); e.hasMoreElements();) {
                   MenuInfo info = (MenuInfo)e.nextElement();

                   if (info.level == 0) 
	             level0Vec.addElement(info);
                   else if (info.level == 1) 
	             level1Vec.addElement(info);
                   else if (info.level == 2) 
        	     level2Vec.addElement(info);
                   else if (info.level == 3) 
	             level3Vec.addElement(info);
                }  
                topstr = "Filter(2d)";
                break;

             case VECTOR:
                for (Enumeration e = svServer.menuHandler.get4dElements(); e.hasMoreElements();) {
                   MenuInfo info = (MenuInfo)e.nextElement();

                   if (info.level == 0) 
	             level0Vec.addElement(info);
                   else if (info.level == 1) 
	             level1Vec.addElement(info);
                   else if (info.level == 2) 
        	     level2Vec.addElement(info);
                   else if (info.level == 3) 
                     level3Vec.addElement(info);
                }  
                topstr = "Filter(Vector)";
                break;
             default:
                return;
          }
          
          DefaultMutableTreeNode top = new DefaultMutableTreeNode(topstr);

          DefaultMutableTreeNode zeroth;
          DefaultMutableTreeNode first;
	  DefaultMutableTreeNode second;
          DefaultMutableTreeNode third;

          for (Enumeration e=level0Vec.elements(); e.hasMoreElements();) {
             MenuInfo lv0item = (MenuInfo)e.nextElement();

             if (!lv0item.isItem) {
                 top.add(zeroth = new DefaultMutableTreeNode(lv0item.name));

                 for (Enumeration e1=level1Vec.elements(); e1.hasMoreElements();) {
                    MenuInfo lv1item = (MenuInfo)e1.nextElement();
              
                    if (lv1item.parent == lv0item.id) {
                       if (!lv1item.isItem) {
                          zeroth.add(first = new DefaultMutableTreeNode(lv1item.name));
                    
                          for (Enumeration e2=level2Vec.elements(); e2.hasMoreElements();) {
                             MenuInfo lv2item = (MenuInfo)e2.nextElement();

                             if (lv2item.parent == lv1item.id) {
                                if (!lv2item.isItem) {
                                   first.add(second = new DefaultMutableTreeNode(lv2item.name));

                                   for (Enumeration e3=level3Vec.elements(); e3.hasMoreElements();) {
                                      MenuInfo lv3item = (MenuInfo)e3.nextElement();
                             
                                      if (lv3item.parent == lv2item.id) {
                                         if (lv3item.isItem)  
                                            second.add(third = new DefaultMutableTreeNode(lv3item.name));
                                      }
			           }
			        } else {
                                   first.add(second = new DefaultMutableTreeNode(lv2item.name));
                                } 
			     }
                          }
		       } else {
                          zeroth.add(first = new DefaultMutableTreeNode(lv1item.name));
                       }
		    }
		 }
	     } else {
                top.add(zeroth = new DefaultMutableTreeNode(lv0item.name));
             }
	  }

          setLayout(new BorderLayout());

          final JTree tree = new JTree(top);

          JScrollPane sp = new JScrollPane();
          sp.setPreferredSize(new Dimension(300, 300));
          sp.getViewport().add(tree);
          add(sp, BorderLayout.CENTER);

          MouseListener ml = new MouseAdapter() {
             public void mouseClicked(MouseEvent e) {
                int selRow = tree.getRowForLocation(e.getX(), e.getY());
                TreePath selPath = tree.getPathForLocation(e.getX(), e.getY());
                if(selRow != -1) {
                   StringBuffer tempSpot = new StringBuffer();
                   Object[] path = selPath.getPath();
                   for(int counter=0; counter<path.length; counter++) {
		      if (!((DefaultMutableTreeNode)path[counter]).isLeaf()) {
                         if(counter > 0)
                            tempSpot.append("/");
                         tempSpot.append(path[counter]);
		      }
                   }
                   menuField.setText(tempSpot.toString());

               }
             }
          };
          tree.addMouseListener(ml);
       }
   }

} // end of class





