import java.awt. *;
import java.io. *;
import java.awt.event. *;
import java.util.*;

import javax.swing.*;
import javax.swing.*;
import javax.swing.*;
import javax.swing.*;

import sv.kernel.*;
import sv.util.*;

public class FilterDelete extends JFrame implements ActionListener {

   private FilterIDE ide;  
   svserver svServer;
   TreePanel trees;  
   JTextField menuField;

   // constructor
   FilterDelete(FilterIDE parent) {
      super("Filter Deleter");
      ide = parent;
      svServer = parent.svServer;

      Container container = getContentPane();

      container.setLayout(new BoxLayout(container, BoxLayout.Y_AXIS));
      container.add(trees = new TreePanel());

      JPanel p = new JPanel();
      p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
      p.setBorder(new EmptyBorder(5,5,5,5));
      p.add(new JLabel("Delete: "));
      p.add(menuField = new JTextField());
      container.add(p);

      p = new JPanel();
      p.setLayout(new FlowLayout(FlowLayout.RIGHT));
      Button btn;
      p.add(btn = new Button("Delete"));
      btn.addActionListener(this);
      p.add(btn = new Button("Cancel"));
      btn.addActionListener(this);
      container.add(p);

      validate();
      pack();
   }

   static final int ONED = 1;
   static final int TWOD = 2;
   static final int THREED = 3;
   static final int VECTOR = 4;

   /**
    * overrided method
    */
   public void actionPerformed(ActionEvent evt) {
      String command = evt.getActionCommand();

      if (command.equals("Delete")) { 
         int dimension = ONED;

         String str = menuField.getText();
         if (str != null) str = str.trim();
         Vector pathVec = new Vector();

         if (str != null && str.length() > 1) {
            StringTokenizer st = new StringTokenizer(str, "/");
            while (st.hasMoreTokens()) 
               pathVec.addElement(st.nextToken());
   
            str = (String)pathVec.firstElement();
            if (str.indexOf("Filter(1d)") >= 0) {
               dimension = ONED;
               pathVec.removeElementAt(0);
            } else if (str.indexOf("Filter(2d)") >= 0) {
               dimension = TWOD;
               pathVec.removeElementAt(0);
            } else if (str.indexOf("Filter(3d)") >= 0) {
               ide.ide.message("Not support yet.");
               return;
            } else if (str.indexOf("Filter(Vector)") >= 0) {
               dimension = VECTOR;
               pathVec.removeElementAt(0);
            } else {
               ide.ide.message("Can't know the target rank.");
               return;
            }
	 }

         ide.ide.message("Updating sv_menu file");

         int errcode = svServer.menuHandler.removeItem(dimension, pathVec);
         
         if (errcode == -1) {
            ide.ide.message("Unsupported dimension error.");
            return;
         } else if (errcode == -2) {
            ide.ide.message("Not found filter item error.");
            return;
         } else if (errcode == -3) {
            ide.ide.message("Unexpected filter deletion error.");
            return;
         } else if (errcode == -4) {
            ide.ide.message("Can't remove whole filter menu structure.");
            return;
         } 

         ide.ide.message("Updating filter menu of opened window.");

         Vector frameVec;
         switch (dimension) {       
  	     case ONED:
                frameVec = svServer.fmanager.get1dFrames();
                
                for (int i=0; i<frameVec.size(); i++) {
                    GFPOneD gframe1d = (GFPOneD)frameVec.elementAt(i);
                    if (gframe1d.getDataType() == DSPlane.NORMAL) {
                       ((GFPOneD)gframe1d).updateFilterMenu();
                    }
                }
                break;

             case TWOD:
                frameVec = svServer.fmanager.get2dFrames();

                for (int i=0; i<frameVec.size(); i++) {
                    GFrame2D gframe2d = (GFrame2D)frameVec.elementAt(i);
                    gframe2d.updateFilterMenu();
                }
                break;

             default:
                ide.ide.message("Unsupported dimension error.");
                ide.ide.progressSetValue(0); 
                return;
          }

         ide.ide.message("Filter deletion completed.");

      } else if (command.equals("Cancel")) 
         ide.ide.message("Deletion canceled.");
     
      ide.deleter = null;
      this.dispose();
   }

   class TreePanel extends JPanel {

       TreePanel() {
          
          String roots[] = new String[3];
          Vector[] onedvec = new Vector[4];
          Vector[] twodvec = new Vector[4];
          Vector[] fourdvec = new Vector[4];

          for (int i=0; i<4; i++) {
             onedvec[i] = new Vector();
             twodvec[i] = new Vector();
             fourdvec[i] = new Vector();
          }

          for (Enumeration e = svServer.menuHandler.get1dElements(); e.hasMoreElements();) {
              MenuInfo info = (MenuInfo)e.nextElement();

              if (info.level == 0) 
                 onedvec[0].addElement(info);
              else if (info.level == 1) 
                 onedvec[1].addElement(info);
              else if (info.level == 2) 
       	         onedvec[2].addElement(info);
              else if (info.level == 3) 
                 onedvec[3].addElement(info);
          }  
          roots[0] = "Filter(1d)";


          for (Enumeration e = svServer.menuHandler.get2dElements(); e.hasMoreElements();) {
              MenuInfo info = (MenuInfo)e.nextElement();

              if (info.level == 0) 
                 twodvec[0].addElement(info);
              else if (info.level == 1) 
                 twodvec[1].addElement(info);
              else if (info.level == 2) 
       	         twodvec[2].addElement(info);
              else if (info.level == 3) 
                 twodvec[3].addElement(info);
          }  
          roots[1] = "Filter(2d)";


         for (Enumeration e = svServer.menuHandler.get4dElements(); e.hasMoreElements();) {
              MenuInfo info = (MenuInfo)e.nextElement();

              if (info.level == 0) 
                 fourdvec[0].addElement(info);
              else if (info.level == 1) 
                 fourdvec[1].addElement(info);
              else if (info.level == 2) 
       	         fourdvec[2].addElement(info);
              else if (info.level == 3) 
                 fourdvec[3].addElement(info);
          }  
          roots[2] = "Filter(Vector)";

          DefaultMutableTreeNode zeroth;
          DefaultMutableTreeNode first;
	  DefaultMutableTreeNode second;
          DefaultMutableTreeNode third;

          DefaultMutableTreeNode[] tops = new DefaultMutableTreeNode[3];
          for (int i=0; i<3; i++)
             tops[i] = new DefaultMutableTreeNode(roots[i], true);

          for (Enumeration e=onedvec[0].elements(); e.hasMoreElements();) {
             MenuInfo lv0item = (MenuInfo)e.nextElement();

             if (!lv0item.isItem) {
                 tops[0].add(zeroth = new DefaultMutableTreeNode(lv0item.name));

                 for (Enumeration e1=onedvec[1].elements(); e1.hasMoreElements();) {
                    MenuInfo lv1item = (MenuInfo)e1.nextElement();
              
                    if (lv1item.parent == lv0item.id) {
                       if (!lv1item.isItem) {
                          zeroth.add(first = new DefaultMutableTreeNode(lv1item.name));
                    
                          for (Enumeration e2=onedvec[2].elements(); e2.hasMoreElements();) {
                             MenuInfo lv2item = (MenuInfo)e2.nextElement();

                             if (lv2item.parent == lv1item.id) {
                                if (!lv2item.isItem) {
                                   first.add(second = new DefaultMutableTreeNode(lv2item.name));

                                   for (Enumeration e3=onedvec[3].elements(); e3.hasMoreElements();) {
                                      MenuInfo lv3item = (MenuInfo)e3.nextElement();
                             
                                      if (lv3item.parent == lv2item.id) {
                                         if (lv3item.isItem)  
                                            second.add(third = new DefaultMutableTreeNode(lv3item.name));
                                      }
			           }
			        } else {
                                   first.add(second = new DefaultMutableTreeNode(lv2item.name));
                                } 
			     }
                          }
		       } else {
                          zeroth.add(first = new DefaultMutableTreeNode(lv1item.name));
                       }
		    }
		 }
	     } else {
                tops[0].add(zeroth = new DefaultMutableTreeNode(lv0item.name));
             }
	  }


          for (Enumeration e=twodvec[0].elements(); e.hasMoreElements();) {
             MenuInfo lv0item = (MenuInfo)e.nextElement();

             if (!lv0item.isItem) {
                 tops[1].add(zeroth = new DefaultMutableTreeNode(lv0item.name));

                 for (Enumeration e1=twodvec[1].elements(); e1.hasMoreElements();) {
                    MenuInfo lv1item = (MenuInfo)e1.nextElement();
              
                    if (lv1item.parent == lv0item.id) {
                       if (!lv1item.isItem) {
                          zeroth.add(first = new DefaultMutableTreeNode(lv1item.name));
                    
                          for (Enumeration e2=twodvec[2].elements(); e2.hasMoreElements();) {
                             MenuInfo lv2item = (MenuInfo)e2.nextElement();

                             if (lv2item.parent == lv1item.id) {
                                if (!lv2item.isItem) {
                                   first.add(second = new DefaultMutableTreeNode(lv2item.name));

                                   for (Enumeration e3=twodvec[3].elements(); e3.hasMoreElements();) {
                                      MenuInfo lv3item = (MenuInfo)e3.nextElement();
                             
                                      if (lv3item.parent == lv2item.id) {
                                         if (lv3item.isItem)  
                                            second.add(third = new DefaultMutableTreeNode(lv3item.name));
                                      }
			           }
			        } else {
                                   first.add(second = new DefaultMutableTreeNode(lv2item.name));
                                } 
			     }
                          }
		       } else {
                          zeroth.add(first = new DefaultMutableTreeNode(lv1item.name));
                       }
		    }
		 }
	     } else {
                tops[1].add(zeroth = new DefaultMutableTreeNode(lv0item.name));
             }
	  }

          for (Enumeration e=fourdvec[0].elements(); e.hasMoreElements();) {
             MenuInfo lv0item = (MenuInfo)e.nextElement();

             if (!lv0item.isItem) {
                 tops[2].add(zeroth = new DefaultMutableTreeNode(lv0item.name));

                 for (Enumeration e1=fourdvec[1].elements(); e1.hasMoreElements();) {
                    MenuInfo lv1item = (MenuInfo)e1.nextElement();
              
                    if (lv1item.parent == lv0item.id) {
                       if (!lv1item.isItem) {
                          zeroth.add(first = new DefaultMutableTreeNode(lv1item.name));
                    
                          for (Enumeration e2=fourdvec[2].elements(); e2.hasMoreElements();) {
                             MenuInfo lv2item = (MenuInfo)e2.nextElement();

                             if (lv2item.parent == lv1item.id) {
                                if (!lv2item.isItem) {
                                   first.add(second = new DefaultMutableTreeNode(lv2item.name));

                                   for (Enumeration e3=fourdvec[3].elements(); e3.hasMoreElements();) {
                                      MenuInfo lv3item = (MenuInfo)e3.nextElement();
                             
                                      if (lv3item.parent == lv2item.id) {
                                         if (lv3item.isItem)  
                                            second.add(third = new DefaultMutableTreeNode(lv3item.name));
                                      }
			           }
			        } else {
                                   first.add(second = new DefaultMutableTreeNode(lv2item.name));
                                } 
			     }
                          }
		       } else {
                          zeroth.add(first = new DefaultMutableTreeNode(lv1item.name));
                       }
		    }
		 }
	     } else {
                tops[2].add(zeroth = new DefaultMutableTreeNode(lv0item.name));
             }
	  }


          setLayout(new BorderLayout());

          DefaultMutableTreeNode superroot = new DefaultMutableTreeNode("Filters");
          for (int i=0; i<3; i++)
             superroot.add(tops[i]);

          final JTree tree = new JTree(superroot);
          tree.setRootVisible(false);

          JScrollPane sp = new JScrollPane();
          sp.setPreferredSize(new Dimension(300, 300));
          sp.getViewport().add(tree);
          add(sp, BorderLayout.CENTER);

          MouseListener ml = new MouseAdapter() {
             public void mouseClicked(MouseEvent e) {
                int selRow = tree.getRowForLocation(e.getX(), e.getY());
                TreePath selPath = tree.getPathForLocation(e.getX(), e.getY());
                if(selRow != -1) {
                   StringBuffer tempSpot = new StringBuffer();
                   Object[] path = selPath.getPath();

                   for(int counter=1; counter<path.length; counter++) {
                      if(counter > 1)
                         tempSpot.append("/");
                      tempSpot.append(path[counter]);
                   }
                   menuField.setText(tempSpot.toString());

               }
             }
          };
          tree.addMouseListener(ml);
       }
   }

} // end of class





