import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.*;

import javax.swing.*;
import javax.swing.*;
import javax.swing.*;

import sv.kernel.*;
import sv.util.*;
import sv.ui.*;

/** 
 * GFrame3D.java
 * Define a frame to draw the 3D graph
 * @version 1.0    10/15/97
 * @author Byeongseob Ki
 */

public class GFrame3D extends GFrame 
                            implements ItemListener, WindowListener {

   //////////////////////////////////////////////////////////////////
   // camera position related variables
   public int prevx, prevy;

   Matrix3D amat = new Matrix3D(), 
            tmat = new Matrix3D();


   public float xcam0 = 0.0f;
   public float ycam0 = 0.0f;
   public float zcam0 = 50.0f;

   public float prevxcam = 0.0f;
   public float prevycam = 0.0f;
   public float prevzcam = 50.0f;
  /////////////////////////////////////////////////////////////////////

  svserver svServer;

  // animation class
  private Animator animator; 

  // animation speed
  private int animationSpeed = 300;

  // Thread to creat GIF image
  private GIFer gifer;

  // to access GCanvas3D class
  GCanvas3D canvas;

  // isosurface threshold selection
  IsoValueSelector selector;

  CutCopyManager editManager;

  // various MenuItems
  private MenuItem animateMI, stopMI, nextMI, prevMI, 
                   thresholdMI, disconMI, splitMI;
  public CheckboxMenuItem checkMI, wireMI, shadingMI, fillMI, boundMI,
                   xgridMI, ygridMI, zgridMI;        


  // configuration
  public Config3D config;

  JSlider zoomer;
  Label zoomValue;
  int zfactor = 20;

  TriObject[] isoObjects;  

  public int received_grid_data;

  /** 
   * to display the information of the graph and error message
   */
  public SimpleMonitor monitor;

  /**
   * constructor
   */                             
  public GFrame3D(String title, DataSet3D iso, 
                      Config3D config, svserver svServer ) {
     super(title);
     dataSetVector.addElement(iso);
     this.config = config;
     this.svServer = svServer;
     dimension_ = DataSet.THREED;
       
     makeMenu();

     // initialize timeArr
     noOfTime = iso.getnoOfTime();
     timeArr = iso.getTimeArr();

     boolean b = false;

     if (config.colormap != null) {
         b = readColorMap(config.colormap);

         if (b) {
           float value;

           for (int i=0; i<101; i++) {
              value = rspline.computePoint((float)i/100f); 
              if (value > 1.0f) value = 1.0f;
              else if (value < 0.0f) value = 0.0f;
              rcolors[i] = value;

              value = gspline.computePoint((float)i/100f);    
              if (value > 1.0f) value = 1.0f;
              else if (value < 0.0f) value = 0.0f;
              gcolors[i] = value;

              value = bspline.computePoint((float)i/100f);    
              if (value > 1.0f) value = 1.0f;
              else if (value < 0.0f) value = 0.0f;
              bcolors[i] = value;
           }
         }
     }

     if (config.colormap == null || b == false) {
         int nControls = 5;
         float[] xcontrols = { 0.0f, 0.25f, 0.5f, 0.75f, 1.0f };
         float[] ycontrols = { 0.0f, 0.5f, 1.0f, 0.5f, 0.0f };
         rspline = new NewtonMethod(xcontrols, ycontrols);
         hspline = new NewtonMethod(xcontrols, ycontrols);

         for (int i=0; i<5; i++)
           ycontrols[i] = 0.5f;
         gspline = new NewtonMethod(xcontrols, ycontrols);
         sspline = new NewtonMethod(xcontrols, ycontrols);

         ycontrols[0] = 1.0f;
         ycontrols[1] = 0.5f; 
         ycontrols[2] = 0.0f;
         ycontrols[3] = 0.5f;
         ycontrols[4] = 1.0f;
         bspline = new NewtonMethod(xcontrols, ycontrols);
         lspline = new NewtonMethod(xcontrols, ycontrols);
     }

     makeObjects();
     setLayout(new BorderLayout());
     canvas = new GCanvas3D(this);

     add("Center", canvas);

     Image img = svServer.loadImageIcon(getClass().getResource("icons/wheel.gif"), "wheel").getImage();
     add("East", new Wheel(Wheel.VERTICAL, 140, canvas, img));

     Panel p2 = new Panel();
     p2.setLayout(new BorderLayout());

     JPanel p = new JPanel();
     p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
     p.add(Box.createRigidArea(new Dimension(20, 1)));
     p.add(new Wheel(Wheel.HORIZONTAL, 140, canvas, img));
     p.add(Box.createRigidArea(new Dimension(20, 1)));

     Panel zp = new Panel();
     zp.setLayout(new BorderLayout());
     zp.add("West", new Label("Zoom:"));
     zp.add("Center", zoomer = new JSlider(JSlider.HORIZONTAL, 0, 100, zfactor));   
     zp.add("East",  zoomValue = new Label("1.0"));
     zoomer.putClientProperty( "JSlider.isFilled", Boolean.TRUE );
     p.add(zp);
     p.add(Box.createRigidArea(new Dimension(20, 1)));
     createSlideListener();

     p2.add("North", p);
     p2.add("Center", monitor = new SimpleMonitor());
     add("South", p2);
     
     monitor.display(title + " created.", false);

     validate();
     addWindowListener(this); 
   }


  /**
   * add another 3D data set to this frame
   * @param dataSet DataSet to be added
   */
  public void addDataSet(DataSet3D dataSet) {
     // reset all the currentTime value to 0
     currentTime = 0;

     dataSetVector.addElement(dataSet);
     updateTimeArr();
     splitMI.setEnabled(true);
     makeObjects();
     canvas.repaint();
     monitor.display("Added new DataSet " + dataSet.getTitle(), false);
     setFrameTitle(dataSet.getTitle() + "...");
  }

  /**
   * update grid information
   * @param val new extraction value
   */
  public void updateGridInfo(int val) {
     monitor.display("Recalculate " + val, false);
     for (Enumeration e=dataSetVector.elements(); e.hasMoreElements();) {
        DataSet3D data = (DataSet3D)e.nextElement();

        int extraction =  data.timeData[0].surfaceGrids.getExtraction();

        int skipmode = DataSet3D.XSKIPED;
        if (xgridMI.getState())
           skipmode = DataSet3D.XSKIPED;
        else if (ygridMI.getState())
           skipmode = DataSet3D.YSKIPED;
        else if (zgridMI.getState())
           skipmode = DataSet3D.ZSKIPED;
        else 
           return;

        int dimx = data.timeData[0].getDimX();
        int dimy = data.timeData[0].getDimY();
        int dimz = data.timeData[0].getDimZ();

        extraction = extraction + val;

        if (skipmode == DataSet3D.XSKIPED) {
           if (extraction < 0) extraction = 0;
           if (extraction > (dimx-1)) extraction = dimx-1;

           if (data.sourceData == null) {
             gridDataRequest(DataSet3D.XSKIPED, extraction, data);
           } else {
              for (int i=0; i<data.getnoOfTime(); i++) 
                 data.extractSurfaceData(data.timeData[i], DataSet3D.XSKIPED, extraction);
	   }

        } else if (skipmode == DataSet3D.YSKIPED) {
           if (extraction < 0) extraction = 0;
           if (extraction > (dimy-1)) extraction = dimy-1;

           if (data.sourceData == null) {
             gridDataRequest(DataSet3D.YSKIPED, extraction, data);
           } else {
              for (int i=0; i<data.getnoOfTime(); i++) 
                 data.extractSurfaceData(data.timeData[i], DataSet3D.YSKIPED, extraction);
	   }
        } else if (skipmode == DataSet3D.ZSKIPED) {
           if (extraction < 0) extraction = 0;
           if (extraction > (dimz-1)) extraction = dimz-1;
        
           if (data.sourceData == null) {
             gridDataRequest(DataSet3D.YSKIPED, extraction, data);
           } else {
              for (int i=0; i<data.getnoOfTime(); i++) 
                 data.extractSurfaceData(data.timeData[i], DataSet3D.ZSKIPED, extraction);
	   }

        } 
     }
     makeObjects();
  }

  /**
   * update dataSet to this frame
   * @param pos position of DataSet to be replaced
   * @param dataSet DataSet to be updated
   */
  public synchronized void updateDataSet(int pos, DataSet3D dataSet, 
                                                           TimeData3D td) {
     dataSetVector.removeElementAt(pos);
     dataSetVector.addElement(dataSet);
     float[] oldtimes = new float[noOfTime];
     System.arraycopy(timeArr, 0, oldtimes, 0, noOfTime);
     updateTimeArr();

     makeObjects(oldtimes, td);

     if (currentTime != noOfTime-1) {
        int saved = currentTime;
        currentTime = 0;

        // send to last timetd.getTime()
        for (int i=0; i<saved; i++)
           next(false);
        next(true);   // one more time
     }

     monitor.display("Updated DataSet " + dataSet.getTitle(), false);
  }


  /**
   * update dataSets to this frame
   */
  public void updateDataSet() {
     makeObjects();
     canvas.repaint();
  }


  /**
   * remove dataSet from this frame
   * @param str DataSet title to be removed
   */
  public void removeDataSet(String str) {
     
    for (Enumeration e=dataSetVector.elements(); e.hasMoreElements();) {
       DataSet3D data = (DataSet3D)e.nextElement();

       if (str.equals(data.getTitle())) {
          removeDataSet(data);
          break;
       }
    } 
    
  }

  /**
   * remove a dataSet from this frame
   * @param dataSet DataSet to be removed
   */
  public void removeDataSet(DataSet3D dataSet) {
     dataSetVector.removeElement(dataSet);

     // reset all the currentTime value to 0
     currentTime = 0;

     if (dataSetVector.size() <= 0) {
        svServer.fmanager.removeNotify(id, DataSet.THREED);
        this.dispose();
     } else {
         DataSet3D data = (DataSet3D)dataSetVector.firstElement();
         if (dataSetVector.size() == 1) {
            setFrameTitle(data.getTitle());
            splitMI.setEnabled(false);
         } else
            setFrameTitle(data.getTitle() + "...");
     }

     updateTimeArr();
     makeObjects();
     canvas.repaint();
     monitor.display("Removed DataSet " + dataSet.getTitle(), false);
  }

 
  /**
   * do additional work after get the user input
   * @param command command to do work
   * @param input user input
   */
  public void doWork(int command, String input) {

    if (command == CommandSet.SAVEAS) {
       boolean saveall = false;

       DataSet3D iso = (DataSet3D)dataSetVector.firstElement();
       if (iso.sourceData == null) {
          int result = JOptionPane.showConfirmDialog(this, 
                             "Save whole data?");
          if(result == JOptionPane.YES_OPTION) {
             saveall = true;
          } else if(result == JOptionPane.NO_OPTION) {
	    // do nothing here
          } else {
             return;
          }
       }

       FileDialog openFileDialog = new FileDialog(this,
                    "Choose data set file name", FileDialog.SAVE);
       openFileDialog.show();
       input = openFileDialog.getFile();
       if (input == null) return;

       boolean b = input.indexOf(".") >=0 ? true : false; 
       if (!b)  input += ".svb";
       input = openFileDialog.getDirectory() + input;

       if (saveall) 
          writeToFile2(input);
       else 
          writeToFile(input);

    } else if (command == CommandSet3D.PRINT) {
        PrintJob printjob =  getToolkit().getPrintJob(this,
                                    "SciVis 3D Plot", printerprops);
                                                        
        if (printjob != null) {          
             Graphics pg = printjob.getGraphics();
             if (pg != null) {
                  canvas.printAll(pg);
                  pg.dispose();         // flush page
             }
             printjob.end();
        }
    
    } else if (command == CommandSet3D.GIFSEQUENCE) {
        FileDialog openFileDialog = new FileDialog(this,
                      "GIF Sequences save", FileDialog.SAVE);
        openFileDialog.show();

        String fname = openFileDialog.getFile();
        if (fname == null) return;
        fname = openFileDialog.getDirectory() + fname;

        if (animator != null) {
           monitor.display("Animation stoped ...", false);
           animator.stop();
           animator = null;
        }

        if (gifer != null) gifer.stop();

        gifer = new GIFer(this, fname);
        gifer.start();

    } else if (command == CommandSet.CUT) {
        doCutCopy(true);
    } else if (command == CommandSet.COPY) {
        doCutCopy(false);
    } else if (command == CommandSet.PASTE) {
        Vector clipboardVec = svServer.clipboard.get3DClipboard();

        int sz = clipboardVec.size();
        if (sz > 0) {
           for (int i=0; i<sz; i++) {
              DataSet3D data = (DataSet3D)clipboardVec.elementAt(i);
              addDataSet(data);
              if (!stealthMode)
                 svServer.cmanager.broadcastDataSet(data, id);
           }
        }
    }  else if (command == CommandSet3D.GOTO) {
        String result = JOptionPane.showInputDialog(this, 
                             "Enter the time data index to go: ");
        if (result == null) return;
        String[] args = new String[1];
        args[0] = result.trim();
        svServer.fmanager.action3d(this, CommandSet3D.GOTO, args,true);

    } else if (command == CommandSet3D.COLORMAP) {

        ColorMapDialog colorDialog = new ColorMapDialog(this);
        colorDialog.show();

    } else if (command == CommandSet3D.THRESHOLD) {
        if (dataSetVector.size() > 1) {
           monitor.display("Does not support multiple DataSets.", true);
           return; 
        }
    
        DataSet3D iso = (DataSet3D)dataSetVector.firstElement();
        if (selector == null) {
           selector = new IsoValueSelector(this, 
                                iso.timeData[currentTime].getThreshold(),
                                iso.timeData[currentTime].getzmin(), 
                                iso.timeData[currentTime].getzmax());
           selector.show();
        }

    } else if (command == CommandSet3D.DISCONNECT) {
        sendDisconnectionCommand(true);
    }
  }

  /**
   * do animation command
   */
  public void doAnimate() {
     if (animator == null) {
        animator = new Animator(this, animationSpeed);
        animator.start();
     } else 
        System.out.println("Animation thread was corrupted ...");

     animateMI.setEnabled(false);
     nextMI.setEnabled(false);
     prevMI.setEnabled(false);
     stopMI.setEnabled(true);
  
  }

  /**
   * split command
   */
  public void doSplit() {

     DataSet3D data = (DataSet3D)dataSetVector.lastElement();

     long frameID  = svServer.fmanager.register(data, -1, null);
     removeDataSet(data);
     canvas.repaint();
  }

  /**
   * do animation  stop command
   */
  public void doStop() {
     if (animator != null) {
       animator.stop();
       animator = null;
     }
     animateMI.setEnabled(true);
     stopMI.setEnabled(false);
     nextMI.setEnabled(true);
     prevMI.setEnabled(true);
  }

  /** 
   * get next DataSet
   * @param done if true, repaint canvas
   */
  public void next(boolean done) {

    currentTime++;
    if (currentTime == noOfTime)
       currentTime = 0;

    if (done) 
       canvas.repaint();
  }
  
  /** 
   * get Previous DataSet (for Animation)
   */
  public void previous() {

    currentTime--;
    if (currentTime == -1)
       currentTime = noOfTime-1;

    canvas.repaint();
  }



  // Cut and copy command
  private void doCutCopy(boolean cutting) {
     Vector clipboardVec = new Vector();

     if (dataSetVector.size() == 1) {
        DataSet3D data = (DataSet3D)dataSetVector.firstElement();
        clipboardVec.addElement(data);

        if (clipboardVec.size() > 0) {
           svServer.clipboard.set3DClipboard(clipboardVec);

           if (cutting) {
              removeDataSet(data);       
              svServer.cmanager.broadcastRemoveDataSet(id, data);
           }
	}
           
     } else {
       if (editManager == null) 
         editManager = 
              new CutCopyManager(this, "Cut & Copy Selection", cutting, 3);
       editManager.show();
     }
  }

  /**
   * do multiple cut and copy
   */
  public void multiCutCopy(int[] selected, boolean cutting) {
     String str = null;
     DataSet3D data;
     Vector clipboardVec = new Vector();

     if (selected.length == 1) {
        data = (DataSet3D)(dataSetVector.elementAt(selected[0]));
        clipboardVec.addElement(data);
     } else if (selected.length > 1) {
        for (int i=0; i<selected.length; i++) {
           data = (DataSet3D)(dataSetVector.elementAt(selected[i]));
           clipboardVec.addElement(data);
        }
     } 
         
     if (clipboardVec.size() > 0) 
        svServer.clipboard.set3DClipboard(clipboardVec);

     if (selected.length == 1) {
        data = (DataSet3D)(dataSetVector.elementAt(selected[0]));
        if (cutting) {
           removeDataSet(data);       
           svServer.cmanager.broadcastRemoveDataSet(id, data);
        }

     } else if (selected.length > 1) {
        for (int i=selected.length-1; i>=0; i--) {
           data = (DataSet3D)(dataSetVector.elementAt(selected[i]));
           if (cutting) {
              removeDataSet(data);  
              svServer.cmanager.broadcastRemoveDataSet(id, data);
           }
        }
     } 
  }



  /**
   * action for boolean arguments
   */
  public void doAction_1(int id, String arg) {
     boolean b;
     if (arg.equals("true"))
        b = true;
     else
        b = false;

     switch (id) {
        case CommandSet3D.WIRE:
           wireMI.setState(b);
           for (int i=0; i<noOfTime; i++)
              isoObjects[i].setWireMode(b);
           break;

        case CommandSet3D.FILL:
           fillMI.setState(b);
           for (int i=0; i<noOfTime; i++)
              isoObjects[i].setFillMode(b);
           break;

        case CommandSet3D.SHADING:
           shadingMI.setState(b);

           if (b && !fillMI.getState()) {
              fillMI.setState(b);
              for (int i=0; i<noOfTime; i++)
                 isoObjects[i].setFillMode(b);
           }

           for (int i=0; i<noOfTime; i++)
              isoObjects[i].setShadingMode(b);
           break;

        case CommandSet3D.GMINMAX:
           System.out.println("Global min max mode is not supported ...");
           break;

     }
     canvas.repaint();
  }



  /**
   * action for makeObjects
   */
  public void doAction_2(int id, String arg) {
     boolean b;
     if (arg.equals("true"))
        b = true;
     else
        b = false;

     DataSet3D iso = (DataSet3D)dataSetVector.firstElement();

     switch (id) {
        case CommandSet3D.BOUNDBOX:
           boundMI.setState(b);
           makeObjects();
           canvas.repaint();
           break;

        case CommandSet3D.XGRID:
           xgridMI.setState(b);
           if (!xgridMI.getState()) break;
           else {
              ygridMI.setState(false);
              zgridMI.setState(false);
           }

	   if (iso.sourceData == null) {
              gridDataRequest(DataSet3D.XSKIPED, (int)(iso.timeData[0].getDimX()/2), iso);
           } else {
              for (Enumeration e=dataSetVector.elements(); e.hasMoreElements();) {
                 DataSet3D data = (DataSet3D)e.nextElement();
                 for (int i=0; i<data.getnoOfTime(); i++) 
                    data.extractSurfaceData(data.timeData[i], DataSet3D.XSKIPED, (int)(iso.timeData[0].getDimX()/2));
              }
              makeObjects();
              canvas.repaint();
	   }
           break;

        case CommandSet3D.YGRID:
           ygridMI.setState(b);
           if (!ygridMI.getState()) break;
           else {
              xgridMI.setState(false);
              zgridMI.setState(false);
           }

           if (iso.sourceData == null) {
              gridDataRequest(DataSet3D.YSKIPED, (int)(iso.timeData[0].getDimY()/2), iso);
           } else {
              for (Enumeration e=dataSetVector.elements(); e.hasMoreElements();) {
                 DataSet3D data = (DataSet3D)e.nextElement();
                 for (int i=0; i<data.getnoOfTime(); i++) 
		   data.extractSurfaceData(data.timeData[i], DataSet3D.YSKIPED, (int)(iso.timeData[0].getDimY()/2));
	      }
              makeObjects();
              canvas.repaint();
           }
           break;

        case CommandSet3D.ZGRID:
           zgridMI.setState(b);
           if (!zgridMI.getState()) break;
           else {
              xgridMI.setState(false);
              ygridMI.setState(false);
           }

           if (iso.sourceData == null) {
              gridDataRequest(DataSet3D.ZSKIPED, (int)(iso.timeData[0].getDimZ()/2), iso);
           } else {
              for (Enumeration e=dataSetVector.elements(); e.hasMoreElements();) {
                 DataSet3D data = (DataSet3D)e.nextElement();
                 for (int i=0; i<data.getnoOfTime(); i++) 
                    data.extractSurfaceData(data.timeData[i], DataSet3D.ZSKIPED, (int)(iso.timeData[0].getDimZ()/2));
	      }
              makeObjects();
              canvas.repaint();
           }
           break;
     }
    
  }


  /**
   * goto action
   */
  public void doGotoAction(String input) {
      int whereto = Integer.valueOf(input.trim()).intValue();

      if ((whereto > noOfTime) || (whereto <= 0))
         monitor.display("Invalid range ...", true);

      int index = 0;
      while (whereto != currentTime) {
         next(false);

         if (index > noOfTime) break;
         index++;
       }
      previous();
  }



  /**
   * kill this frame
   */
  public void killFrame() {

     if (animator != null) {
       animator.stop();
       animator = null;
     } 
     if (gifer != null) {
       gifer.stop();
       gifer = null;
     } 
     sendDisconnectionCommand(false);
     svServer.fmanager.removeNotify(id, DataSet.THREED);
     this.dispose();
  }


  /**
   * set color map
   */
  public void setColorMap(String[] str) {
     super.setColorMap(str);
     makeObjects();
     canvas.repaint();
  }

  /**
   * change color map info
   */
  public void changeColorMap(String[] args) {
     svServer.fmanager.action3d(this, CommandSet.COLORMAPCHANGE, args, true);
  }

  // send disconnection command
  private void sendDisconnectionCommand(boolean b) {
    
    try {
       for (int i=0; i<dataSetVector.size(); i++) {
          DataSet3D iso = (DataSet3D)dataSetVector.elementAt(i);
         Socket sock = new Socket(iso.getSourceHostname(), iso.getSourcePort());
         DataOutputStream output = new DataOutputStream(sock.getOutputStream());

         if (b) {
            output.writeInt(2);                  // send command (really quit)
            return;
         }
         output.writeInt(4);                     // remove cache files
         output.writeInt(iso.getVersion());      // send data_version
       }
        
    } catch (Exception e) {
       monitor.display("Connection failed to isosurface server....", true);
    }    

  }

  // send new threshold 
  public void setNewThreshold(float val, boolean b, DataSet3D iso) {

    if ((val > iso.timeData[currentTime].getzmax()) || 
                  (val < iso.timeData[currentTime].getzmin())) {
       monitor.display("Invalid range ...", true);
       return;
    }

    try {
       Socket sock = new Socket(iso.getSourceHostname(), iso.getSourcePort());
       DataOutputStream output = new DataOutputStream(sock.getOutputStream());

       output.writeInt(3);                     // send command
       output.writeInt(iso.getVersion());      // send data version

       if (b) {                                // only for one time data
           output.writeInt(1);
           output.writeFloat(iso.timeData[currentTime].getTime());
       } else
          output.writeInt(0);

       output.writeFloat(val);              // send new threshold
        
    } catch (Exception e) {
       monitor.display("Connection failed to isosurface server....", true);
    }    
  }



  // request grid data infomation
  public void gridDataRequest(int orient, int extraction, DataSet3D iso) {
    received_grid_data = 0;

    try {
       Socket sock = new Socket(iso.getSourceHostname(), iso.getSourcePort());
       DataOutputStream output = new DataOutputStream(sock.getOutputStream());

       output.writeInt(5);                     // send command

       output.writeInt(iso.getVersion());      // send data version
    
       // write data set name
       byte[] byteArr = iso.getTitle().getBytes();
       output.writeInt(byteArr.length); 
       output.write(byteArr, 0, byteArr.length);

       output.writeInt(orient);                // send orientation
       output.writeInt(extraction);            // send extraction value
        
    } catch (Exception e) {
       monitor.display("Connection failed to isosurface server....", true);
    }    
  }


  // write all objects of this frame to a file
  public void writeToFile(String fname) {
     writeToFileReal(fname, 0);
  }


  // write all objects of this frame to a file including whole data
  public void writeToFile2(String fname) {
     writeToFileReal(fname, 1);
     if (fname.endsWith(".svb")) {
        String newfname = fname.substring(0, fname.length()-1);
        newfname = newfname + "w";
	//System.out.println("Whole data set file name: " + newfname);

        DataSet3D iso = (DataSet3D)dataSetVector.firstElement();
        try {
           Socket sock = new Socket(iso.getSourceHostname(), 
                                             iso.getSourcePort());
           DataOutputStream output = 
                       new DataOutputStream(sock.getOutputStream());

           output.writeInt(6);                     // send command
           output.writeInt(iso.getVersion());      // send data version

           // write file name
           byte[] byteArr = newfname.getBytes();
           output.writeInt(byteArr.length); 
           output.write(byteArr, 0, byteArr.length);

        } catch (Exception e) {
           monitor.display("Connection failed to isosurface server....", true);
        }    

     } else {
        System.out.println("Not support without extension \".svb\"");
     }
     
  }


  private void writeToFileReal(String fname, int info) {
     DataOutputStream os = null;
     byte[] byteArr;
     int sz = dataSetVector.size();
   
     try {
        os = new DataOutputStream(
               new BufferedOutputStream(new FileOutputStream(fname)));
     } catch (Exception e) {
        System.out.println("File writing error(3d) ...");
        return;
     }

     try {
        // write dimension
        os.writeInt(DataSet.THREED);
        
        // write frame title
        byteArr = frameTitle.getBytes();
        os.writeInt(byteArr.length); 
        os.write(byteArr, 0, byteArr.length);
              
        // write configuration stream
        byteArr = config.getFileSettings().getBytes();
        os.writeInt(byteArr.length); 
        os.write(byteArr, 0, byteArr.length);

        // write drawing objects stream (NOT YET IMPLEMENTED)
        os.writeInt(0); 

        // write whole data set information
        os.writeInt(info); 

        // write the number of data sets
        os.writeInt(sz);  

        // now write the data sets
        for (int i=0; i<sz; i++) {
           DataSet3D dataSet = (DataSet3D)dataSetVector.elementAt(i);

           byteArr = dataSet.getTitle().getBytes();
           os.writeInt(byteArr.length); 
           os.write(byteArr, 0, byteArr.length);

           int noOfTimeData = dataSet.timeData.length;
           os.writeInt(noOfTimeData);   // send number of TimeData
           for (int k=0; k<noOfTimeData; k++) 
              dataSet.timeData[k].sendingData(os);
        } 

        os.close();
     } catch (Exception e) {
        System.out.println("Can't write the 3d data to a file ...");
     }     
  }


  // construct the objects
  public synchronized void makeObjects() {

     isoObjects = new TriObject[noOfTime];
     for (int i=0; i<noOfTime; i++) 
        isoObjects[i] = new TriObject();

     int index = 0;
     boolean flag = false;
     for (Enumeration e=dataSetVector.elements(); e.hasMoreElements();) {
         DataSet3D data = (DataSet3D)e.nextElement();

         for (int i=0; i<data.getnoOfTime(); i++) {
            for (int j=0; j<noOfTime; j++) {
              if (data.getTimeValue(i) == timeArr[j]) {

                 int nvert = data.timeData[i].getNVERT();
                 int ntri = data.timeData[i].getNTRI();
                 int oldvert = isoObjects[j].getNVERT();
                 int oldtris = isoObjects[j].getNTRI();
//System.out.println(timeArr[j] + " ---- " + ntri);
                 isoObjects[j].setWireMode(wireMI.getState());
                 isoObjects[j].setFillMode(fillMI.getState());
                 isoObjects[j].setShadingMode(shadingMI.getState());
                 isoObjects[j].setTriangleLimits(ntri, nvert);

                 if (index == 1 && isoObjects[j].colorIndex.size() != 1) {
                    isoObjects[j].colorIndex = new Vector();
                    isoObjects[j].colorIndex.addElement(new Integer(oldtris));
                 } else if (index > 1 && !flag) {
                    Integer saved = (Integer)isoObjects[j].colorIndex.firstElement(); 
                    isoObjects[j].colorIndex = new Vector();
                    isoObjects[j].colorIndex.addElement(saved);
                    isoObjects[j].colorIndex.addElement(new Integer(oldtris)); 
                    flag = true;
                 }

                 for (int iv=0; iv<nvert; iv++) 
                   isoObjects[j].addVert(data.timeData[i].vertex[iv][0], 
                                         data.timeData[i].vertex[iv][1], 
                                         data.timeData[i].vertex[iv][2]);

                 for (int it=0; it<ntri; it++) {
                   int newtri[] =  new int[3];
                   newtri[0] = data.timeData[j].triangles[it][0] + oldvert;
                   newtri[1] = data.timeData[j].triangles[it][1] + oldvert;
                   newtri[2] = data.timeData[j].triangles[it][2] + oldvert;

                   isoObjects[j].addPoly(3, newtri, Color.blue);
                 }

                 if (data.timeData[i].BOUNDBOX_POSSIBLE && boundMI.getState() && ntri > 0) {

                    for (int iv=0; iv<8; iv++) 
                        isoObjects[j].addVert(
                               data.timeData[i].boundingbox[iv][0], 
                               data.timeData[i].boundingbox[iv][1], 
                               data.timeData[i].boundingbox[iv][2]);
                    isoObjects[j].add(nvert,   nvert+1, Color.black);
                    isoObjects[j].add(nvert+1, nvert+2, Color.black);
                    isoObjects[j].add(nvert+2, nvert+3, Color.black);
                    isoObjects[j].add(nvert+3, nvert,   Color.black);
                    isoObjects[j].add(nvert+4, nvert+5, Color.black);
                    isoObjects[j].add(nvert+5, nvert+6, Color.black);
                    isoObjects[j].add(nvert+6, nvert+7, Color.black);
                    isoObjects[j].add(nvert+7, nvert+4, Color.black);
                    isoObjects[j].add(nvert,   nvert+4, Color.black);
                    isoObjects[j].add(nvert+1, nvert+5, Color.black);
                    isoObjects[j].add(nvert+2, nvert+6, Color.black);
                    isoObjects[j].add(nvert+3, nvert+7, Color.black);
                 }

                 int gridpos = nvert;
                 if (data.timeData[i].BOUNDBOX_POSSIBLE && boundMI.getState() && ntri > 0)
                    gridpos += 8;

                 if (data.timeData[i].GRID_AVAILABLE) {

                    GridInfo3D grids = data.timeData[i].surfaceGrids;
                    if (grids == null) 
                       break;
             
                    int dimx = data.timeData[j].getDimX();
                    int dimy = data.timeData[j].getDimY();
                    int dimz = data.timeData[j].getDimZ();      

                    int gridsize = grids.getSize();
                    int extraction = grids.getExtraction();
                    Color gridcolor;

                    if (grids.getSkipmode() == DataSet3D.XSKIPED && xgridMI.getState()) {
                        int end1 = dimy+1, end2 = dimz+1;
                        for (int iy=0; iy<end1; iy++) 
			  for (int iz=0; iz<end2; iz++) 
                             isoObjects[j].addVert(extraction, iy, iz);
      
                        int idx = 0;
                        for (int iy=0; iy<dimy; iy++) {
			  for (int iz=0; iz<dimz; iz++) {
                             int rect[] =  new int[4];
                             int tmppos = iy * end2 + iz;
                             int tmppos2 = (iy+1) * end2 + iz;
                             
                             rect[0] = gridpos + tmppos;
                             rect[1] = gridpos + tmppos2;
                             rect[2] = gridpos + tmppos2 + 1;
                             rect[3] = gridpos + tmppos + 1;

                             tmppos++;
                             tmppos2++;
                             int val = (int)((grids.getValue(idx)-data.timeData[i].getzmin()) / (data.timeData[i].getzmax()-data.timeData[i].getzmin()) * 100f);
                            
                             if (colorModel == GFrame.RGBCOLORMODEL) 
                               gridcolor =
                                 new Color((int)(rcolors[val] * 255f), 
                                           (int)(gcolors[val] * 255f), 
                                           (int)(bcolors[val] * 255f));
                             else 
                                gridcolor =
                                  Color.getHSBColor((float)hcolors[val], 
                                                    (float)scolors[val],
                                                    (float)lcolors[val]);

                             isoObjects[j].addPoly(4, rect, gridcolor);
                             idx++;
                          }
			}

                    } else if (grids.getSkipmode() == DataSet3D.YSKIPED && ygridMI.getState()) {

                        int end1 = dimx+1, end2 = dimz+1;
                        for (int ix=0; ix<end1; ix++) 
			  for (int iz=0; iz<end2; iz++) 
                             isoObjects[j].addVert(ix, extraction, iz);
      
                        int idx = 0;
                        for (int ix=0; ix<dimx; ix++) {
			  for (int iz=0; iz<dimz; iz++) {
                             int rect[] =  new int[4];
                             int tmppos = ix * end2 + iz;
                             int tmppos2 = (ix+1) * end2 + iz;
                             
                             rect[0] = gridpos + tmppos;
                             rect[1] = gridpos + tmppos2;
                             rect[2] = gridpos + tmppos2 + 1;
                             rect[3] = gridpos + tmppos + 1;

                             tmppos++;
                             tmppos2++;
                             int val = (int)((grids.getValue(idx)-data.timeData[i].getzmin()) / (data.timeData[i].getzmax()-data.timeData[i].getzmin()) * 100f);
                            
                             if (colorModel == GFrame.RGBCOLORMODEL) 
                               gridcolor =
                                 new Color((int)(rcolors[val] * 255f), 
                                           (int)(gcolors[val] * 255f), 
                                           (int)(bcolors[val] * 255f));
                             else 
                                gridcolor =
                                  Color.getHSBColor((float)hcolors[val], 
                                                    (float)scolors[val],
                                                    (float)lcolors[val]);

                             isoObjects[j].addPoly(4, rect, gridcolor);
                             idx++;
                          }
			}

                    } else if (grids.getSkipmode() == DataSet3D.ZSKIPED && zgridMI.getState()) {
                        int end1 = dimx+1, end2 = dimy+1;
                        for (int ix=0; ix<end1; ix++) 
			  for (int iy=0; iy<end2; iy++) 
                             isoObjects[j].addVert(ix, iy, extraction);
      
                        int idx = 0;
                        for (int ix=0; ix<dimx; ix++) {
			  for (int iy=0; iy<dimy; iy++) {
                             int rect[] =  new int[4];
                             int tmppos = ix * end2 + iy;
                             int tmppos2 = (ix+1) * end2 + iy;
                             
                             rect[0] = gridpos + tmppos;
                             rect[1] = gridpos + tmppos2;
                             rect[2] = gridpos + tmppos2 + 1;
                             rect[3] = gridpos + tmppos + 1;

                             tmppos++;
                             tmppos2++;
                             int val = (int)((grids.getValue(idx)-data.timeData[i].getzmin()) / (data.timeData[i].getzmax()-data.timeData[i].getzmin()) * 100f);
                            
                             if (colorModel == GFrame.RGBCOLORMODEL) 
                               gridcolor =
                                 new Color((int)(rcolors[val] * 255f), 
                                           (int)(gcolors[val] * 255f), 
                                           (int)(bcolors[val] * 255f));
                             else 
                                gridcolor =
                                  Color.getHSBColor((float)hcolors[val], 
                                                    (float)scolors[val],
                                                    (float)lcolors[val]);

                             isoObjects[j].addPoly(4, rect, gridcolor);
                             idx++;
                          }
			}

                    } // end of ZSKIPED mode
		 }
                 break;
	      }
	    }
         }
         index++;
     }
  }

  
  private synchronized void makeObjects(float[] oldTimes, TimeData3D td) {

     int pos = -1;

     for (int i=0; i<oldTimes.length; i++) {
       if (oldTimes[i] > td.getTime()) {
         pos = i;
         break;
       }
     }

     if (pos < 0)
       pos = noOfTime - 1;

     TriObject dummy[] = new TriObject[noOfTime];
     System.arraycopy(isoObjects, 0, dummy, 0, pos);
     System.arraycopy(isoObjects, pos, dummy, pos+1, noOfTime-pos-1);
     isoObjects = dummy;

     isoObjects[pos] = new TriObject();

     int nvert = td.getNVERT();
     int ntri = td.getNTRI();
     isoObjects[pos].setTriangleLimits(nvert, ntri);

     for (int iv=0; iv<nvert; iv++) 
       isoObjects[pos].addVert(td.vertex[iv][0], td.vertex[iv][1], 
                                                       td.vertex[iv][2]);

     for (int it=0; it<ntri; it++) 
       isoObjects[pos].addPoly(3, td.triangles[it], Color.blue);
  }


  //  make menu 
  private void makeMenu() {

     MenuBar menubar = new MenuBar();
     CommandSet3D command;
     MenuItem mi;

     // File manager
     Menu fileMenu = new Menu("File");

     addMI("Save as ...", fileMenu, new CommandSet3D(this, CommandSet.SAVEAS));
     addMI("Print", fileMenu, new CommandSet3D(this, CommandSet3D.PRINT));
     addMI("GIF Sequences", fileMenu, 
                              new CommandSet3D(this, CommandSet3D.GIFSEQUENCE));
     fileMenu.addSeparator();
     addMI("Quit C-q", fileMenu, new CommandSet3D(this, CommandSet3D.QUIT));

     menubar.add(fileMenu);

     // Cut, Copy and Paste
     Menu editMenu = new Menu("Edit");
     addMI("Cut", editMenu, new CommandSet3D(this, CommandSet.CUT));
     addMI("Copy", editMenu, new CommandSet3D(this, CommandSet.COPY));
     addMI("Paste", editMenu, new CommandSet3D(this, CommandSet.PASTE));

     menubar.add(editMenu);

     Menu configMenu = new Menu("View");

     boundMI = new CheckboxMenuItem("Bound box");
     command = new CommandSet3D(this, CommandSet3D.BOUNDBOX);
     boundMI.addItemListener(command);
     boundMI.setState(config.bounding_box); 
     configMenu.add(boundMI);

     configMenu.addSeparator();

     wireMI = new CheckboxMenuItem("Wire Frame");
     command = new CommandSet3D(this, CommandSet3D.WIRE);
     wireMI.addItemListener(command);
     wireMI.setState(true);
     configMenu.add(wireMI);

     fillMI = new CheckboxMenuItem("Fill Mode");
     command = new CommandSet3D(this, CommandSet3D.FILL);
     fillMI.addItemListener(command);
     fillMI.setState(false);
     configMenu.add(fillMI);

     shadingMI = new CheckboxMenuItem("Shading");
     command = new CommandSet3D(this, CommandSet3D.SHADING);
     shadingMI.addItemListener(command);
     shadingMI.setState(false);
     configMenu.add(shadingMI);

     configMenu.addSeparator();

     Menu colorMenu = new Menu("Colors");

     mi = new MenuItem("Background Color");
     ActionListener bgColorChooser = new ActionListener() {
	public void actionPerformed(ActionEvent e) {
           Color c = 
              JColorChooser.showDialog(GFrame3D.this, 
                                      "Background Color", config.bgColor);
           if (c == null)
               return;
           String args[] = { String.valueOf(c.getRed()), 
                             String.valueOf(c.getGreen()), 
                             String.valueOf(c.getBlue()) };
           svServer.fmanager.action3d(GFrame3D.this, 
                                    CommandSet.BACKCOLOR, args, true);
	}
     };
     mi.addActionListener(bgColorChooser);
     colorMenu.add(mi);

     mi = new MenuItem("Color Map Editor");
     command = new CommandSet3D(this, CommandSet3D.COLORMAP);
     mi.addActionListener(command);
     colorMenu.add(mi);

     configMenu.add(colorMenu);

     checkMI = new CheckboxMenuItem("Use global minmax");
     command = new CommandSet3D(this, CommandSet3D.GMINMAX);
     checkMI.addItemListener(command);
     checkMI.setState(config.global_mode);
     configMenu.add(checkMI);

     menubar.add(configMenu);

     Menu controlMenu = new Menu("Control");

     thresholdMI = new MenuItem("Set Threshold");
     command = new CommandSet3D(this, CommandSet3D.THRESHOLD);
     thresholdMI.addActionListener(command);
     controlMenu.add(thresholdMI);

     disconMI = new CheckboxMenuItem("Disconnect to client");
     command = new CommandSet3D(this, CommandSet3D.DISCONNECT);
     disconMI.addActionListener(command);
     controlMenu.add(disconMI);

     menubar.add(controlMenu);
     DataSet3D dummy = (DataSet3D)dataSetVector.firstElement();
     if (dummy.getSourceHostname() == null) controlMenu.setEnabled(false);

     // Misc menu
     Menu miscMenu = new Menu("Misc");

     // animation menus
     Menu animationMenu = new Menu("Animation");

     animateMI = new MenuItem("Animate C-a");
     command = new CommandSet3D(this, CommandSet3D.ANIMATE);
     animateMI.addActionListener(command);
     animationMenu.add(animateMI);

     stopMI = new MenuItem("Stop C-s");
     command = new CommandSet3D(this, CommandSet3D.STOP);
     stopMI.addActionListener(command);
     animationMenu.add(stopMI);
     stopMI.setEnabled(false);

     nextMI = new MenuItem("Next C-n");
     command = new CommandSet3D(this, CommandSet3D.NEXT);
     nextMI.addActionListener(command);
     animationMenu.add(nextMI);

     prevMI = new MenuItem("Previous C-p");
     command = new CommandSet3D(this, CommandSet3D.PREVIOUS);
     prevMI.addActionListener(command);
     animationMenu.add(prevMI);
     miscMenu.add(animationMenu);

     Menu gridMenu = new Menu("Grids");

     xgridMI = new CheckboxMenuItem("X Grid");
     command = new CommandSet3D(this, CommandSet3D.XGRID);
     xgridMI.addItemListener(command);
     xgridMI.setState(config.xgrid); 
     gridMenu.add(xgridMI);

     ygridMI = new CheckboxMenuItem("Y Grid");
     command = new CommandSet3D(this, CommandSet3D.YGRID);
     ygridMI.addItemListener(command);
     ygridMI.setState(config.ygrid); 
     gridMenu.add(ygridMI);

     zgridMI = new CheckboxMenuItem("Z Grid");
     command = new CommandSet3D(this, CommandSet3D.ZGRID);
     zgridMI.addItemListener(command);
     zgridMI.setState(config.zgrid); 
     gridMenu.add(zgridMI);

     miscMenu.add(gridMenu);

     mi = new MenuItem("Goto C-g");
     command = new CommandSet3D(this, CommandSet3D.GOTO);
     mi.addActionListener(command);
     miscMenu.add(mi);

     // Split the DataSet
     splitMI = new MenuItem("Split DataSets");
     command = new CommandSet3D(this, CommandSet.SPLIT);
     splitMI.addActionListener(command);
     miscMenu.add(splitMI);
     if (dataSetVector.size() < 2) 
        splitMI.setEnabled(false);

     menubar.add(miscMenu);

     setMenuBar(menubar);
  }

  // add menuItem to menu
  private void addMI(String str, Menu parent, CommandSet3D command) {
     MenuItem mi = new MenuItem(str);
     mi.addActionListener(command);
     parent.add(mi);
  }

  /**
   * overrided method (for list)
   */
  public void itemStateChanged(ItemEvent e) {

     String itemName  = e.getItem().toString();

     if (e.getStateChange() == ItemEvent.SELECTED) {
        if (itemName.equals("X")) {
           System.out.println("X");
        } else if (itemName.equals("Y")) {
           System.out.println("Y");
        } else if (itemName.equals("Z")) {
           System.out.println("Z");
        } else if (itemName.equals("XYZ")) {
           System.out.println("XYZ");
        }
     }
  }


  // slide value changes
  private void createSlideListener() {
     ChangeListener changeListener = new ChangeListener() {
        public void stateChanged(ChangeEvent e) {
           zfactor = zoomer.getValue();
           canvas.scalefudge = zfactor / 20.f;
           
           String val = String.valueOf(canvas.scalefudge);
           if (val.length() > 4)
              val = val.substring(0, 4);
           
	   zoomValue.setText(val);
           canvas.repaint();
        }
     };

     zoomer.addChangeListener(changeListener);
  }


  // virtual desktop management
  private boolean iconized = false;
  private boolean activated = true;

  public boolean isIconized() {
     return iconized;
  }

  public boolean isActivated() {
     return activated;
  }

  public void windowOpened(WindowEvent e) {
  }
  public void windowClosing(WindowEvent e) {
  }

  public void windowClosed(WindowEvent e) {
  }

  public void windowIconified(WindowEvent e) {
    iconized = true;
    if (svServer.controller != null)
       svServer.controller.desktopRepaint();
  }

  public void windowDeiconified(WindowEvent e) {
    iconized = false;
    if (svServer.controller != null)
       svServer.controller.desktopRepaint();
  }

  public void windowActivated(WindowEvent e) {
    activated = true;
    if (svServer.controller != null)
       svServer.controller.desktopRepaint();
  }

  public void windowDeactivated(WindowEvent e) {
    activated = false;
    if (svServer.controller != null)
       svServer.controller.desktopRepaint();
  }

} // end of class


