
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import javax.swing.*;

import sv.ui.*;
import sv.kernel.*;
import sv.script.*;

/** 
 * ScriptFrame.java
 * script environment frame
 * @version 1.0   4/15/98 
 * @author Byeongseob Ki
 */

public class ScriptFrame extends Frame implements ActionListener {
  private svserver svServer; 

  ScriptHandler handler;
  
  TextArea console;
  Button loadBtn, closeBtn, verboseBtn, runBtn, clearBtn, contBtn;
  
  public ScriptFrame(svserver svServer) {
     super("Script Console");
     this.svServer = svServer;
     makeGUI();
  }

  /**
   * add text to the console
   */
  public void append(String str) {
     console.append(str);
  }

  /**
   * perform console button action
   */
  public void actionPerformed(ActionEvent e) {

     String command = e.getActionCommand();

     if (command.equals("Load")) {
        FileDialog openFileDialog = new FileDialog(this,
                         "Open script file", FileDialog.LOAD);
        openFileDialog.show();

        String fname = openFileDialog.getFile();
        if (fname == null) return;
        fname = openFileDialog.getDirectory() + fname;

        handler = new ScriptHandler(svServer, this, fname);
        handler.start();

        runBtn.setEnabled(true);
        contBtn.setEnabled(false);

     } else if (command.equals("Continue")) {
        contBtn.setEnabled(false);
        handler.resume();

     } else if (command.equals("Close")) {
        svServer.mainb.scriptBtn.setEnabled(true);
        handler = null;
        this.dispose();

     } else if (command.equals("Run")) {
        runBtn.setEnabled(false);
        handler.resume();

     } else if (command.equals("Clear")) {
        console.setText("");
     
     } else if (command.equals("Verbose")) {
        if (handler != null) {
 	   if (handler.getVerboseOn()) {
              handler.setVerboseOn(false);
              append("\nVerbose mode off.");
           } else {
              handler.setVerboseOn(true);
              append("\nVerbose mode on.");
           }
	} else
           append("\nLoad the script first.");
     }
  }

  private void makeGUI() {
     setLayout(new BorderLayout());

     JPanel btnPanel = new JPanel();
     btnPanel.setBorder(SVUIC.systemBorder);
     btnPanel.setLayout(new GridLayout(2, 3));
     btnPanel.setFont(new Font("Helvetica", Font.BOLD, 12));

     btnPanel.add(loadBtn = new Button("Load"));
     btnPanel.add(runBtn = new Button("Run"));
     btnPanel.add(contBtn = new Button("Continue"));
     btnPanel.add(clearBtn = new Button("Clear"));
     btnPanel.add(verboseBtn = new Button("Verbose"));
     btnPanel.add(closeBtn = new Button("Close"));

     runBtn.setEnabled(false);
     contBtn.setEnabled(false);

     loadBtn.addActionListener(this);
     runBtn.addActionListener(this);
     contBtn.addActionListener(this);
     verboseBtn.addActionListener(this);
     clearBtn.addActionListener(this);
     closeBtn.addActionListener(this);

     add("South", btnPanel);

     add("Center", console = new TextArea("\nScivis script console v1.0"));
     console.setEditable(false);

     validate();
  }

} // end of class
