package sv.kernel;

import java.util.*;
import java.io.*;

import sv.util.*;

/** 
 * FilterMenuHandler.java
 * Merge Filter Handler
 * @version 1.0   10/1/97 
 * @author Byeongseob Ki
 */

public class FilterMenuHandler {

  public static Vector menuVec;          // MenuInfo vector for 1D
  public static Vector menuVec2D ;       // MenuInfo vector for 2D
  public static Vector menuVec4D ;       // MenuInfo vector for 4D
   
  String sv_menu = null;

  public FilterMenuHandler(String sv_menu) {
     menuVec = new Vector();
     menuVec2D = new Vector();
     menuVec4D = new Vector();
 
     if (sv_menu != null) {
       this.sv_menu = sv_menu;
       parseMenuInfo(sv_menu);
     } 
  }

  /**
   * retrun menuVec enumeration
   */
  public Enumeration get1dElements() {
     return menuVec.elements();
  }


  /**
   * retrun menuVec2D enumeration
   */
  public Enumeration get2dElements() {
     return menuVec2D.elements();
  }

  /**
   * retrun menuVec4D enumeration
   */
  public Enumeration get4dElements() {
     return menuVec4D.elements();
  }

  /**
   * insert menu item into the filter menu
   * @return error code 
   */
  public int insertItem(int dim, String classname, String packname, Vector vec) {
      Vector updatevec = null;

      switch (dim) {
         case 1:
            updatevec = menuVec;
            break;
         case 2:
            updatevec = menuVec2D;
            break;
         case 4:
            updatevec = menuVec4D;
            break;
         default:
 	    return -1;               // unsupported dimension
      }

      int level = vec.size();           // level will be at most 4
      int currid = updatevec.size();
      String[] source = new String[level];
      vec.copyInto(source);

      int[] root = new int[3];
      for (int i=0; i<3; i++)
         root[i] = -1;

      for (int i=0; i<level; i++) {
          if (i == (level-1)) {          // <==========  menu item
             for (int j=0; j<updatevec.size(); j++) {
                MenuInfo info = (MenuInfo)updatevec.elementAt(j);
                if (info.name.equals(source[i])) 
                   return -2;            // duplicated item
             }
             if (i == 0)                 // first element is an item
                updatevec.insertElementAt(new MenuInfo(source[i], classname, packname, -1, i, currid, -1), 0);
             else if (root[i-1] == -1)   // missing parent node
	        return -3;                  
             else                        // normally done  
                updatevec.insertElementAt(new MenuInfo(source[i], classname, packname, -1, i, currid, root[i-1]), root[i-1]);

          } else {                       // <==========  menu
             for (int j=0; j<updatevec.size(); j++) {
                MenuInfo info = (MenuInfo)updatevec.elementAt(j);
                if (info.name.equals(source[i])) {
                   root[i] = info.id;
                   break;
                } 
             }
             if (root[i] == -1) {          // Not found, so make menu
                root[i] = ++currid;
                if (i == 0)
                   updatevec.insertElementAt(new MenuInfo(source[i], i, root[i], -1), 0);
                else
		   updatevec.insertElementAt(new MenuInfo(source[i], i, root[i], root[i-1]), root[i-1]);
             }

          }
      }
      writeToFile();

      // reread ...
      if (sv_menu != null) {
          menuVec.removeAllElements();
          menuVec2D.removeAllElements();
          menuVec4D.removeAllElements();
          parseMenuInfo(sv_menu);
      }

      return 0;           // success
  }


  /**
   * delete menu item from the filter menu
   * @return error code 
   */
  public int removeItem(int dim, Vector vec) {
      Vector updatevec = null;
      boolean found = false;

      switch (dim) {
         case 1:
            updatevec = menuVec;
            break;
         case 2:
            updatevec = menuVec2D;
            break;
         case 4:
            updatevec = menuVec4D;
            break;
         default:
 	    return -1;               // unsupported dimension
      }

      int level = vec.size();           // level will be at most 4
      String[] source = new String[level];
      vec.copyInto(source);

      if (level < 1) return -4;         // remove whole filter structure

      // find the menu or menu item that equals to last input element
      String targetStr = source[level-1];

      for (int i=0; i<updatevec.size(); i++) {
         MenuInfo info = (MenuInfo)updatevec.elementAt(i);
         if (info.name.equals(targetStr)) { 
            MenuInfo[] target = new MenuInfo[updatevec.size()];
            updatevec.copyInto(target);
     
            removeChildItem(target, info);   
            target[i] = null;
            // ALSO NEED REARRANGE THE PARENT WHICH HAS NO CHILD

            updatevec.removeAllElements();
            for (int j=0; j<target.length; j++) 
              if (target[j] != null)
                 updatevec.addElement(target[j]);

            found = true;
            break;
         }
      }

      if (!found)                 // not found error
         return -2; 


      writeToFile();

      // reread ...
      if (sv_menu != null) {
          menuVec.removeAllElements();
          menuVec2D.removeAllElements();
          menuVec4D.removeAllElements();
          parseMenuInfo(sv_menu);
      }
      return 0;            // sucesss
  }

  // very recursive remover (children)
  private void removeChildItem(MenuInfo[] target, MenuInfo info) {
     for (int i=0; i<target.length; i++) {
       if (target[i] == null) continue;

       if (target[i].parent == info.id) {      
 	  if (target[i].className != null)      // in case there is subtree
             target[i] = null;
          else                                  // in case leaf
             removeChildItem(target, target[i]);
       } 
     }
  }

  public void writeToFile() {
     try {
        if (sv_menu == null) return;

        File oldfile = new File(sv_menu);
        File oldenough = new File(sv_menu + ".old");
        if (oldenough.exists()) {
           oldenough.delete();
           oldenough = new File(sv_menu + ".old");
	}
        oldfile.renameTo(oldenough);

        PrintWriter writer = new PrintWriter(new FileWriter(sv_menu));
        writer.println("#########################################################################");
        writer.println("# Instruction:");
        writer.println("#");
        writer.println("# 1. Each line should start with begin, <menu>, <item>, </menu>, or end.");
        writer.println("# 2. Each menu allows at most 3 submenu.");
        writer.println("# 3. Keywords NAME, CLASS, TYPE, SHORTKEY must be uppercases.");
        writer.println("#      NAME - menu name");
        writer.println("#      CLASS - class name");
        writer.println("#      PACKAGE - package name of your filter. You have to include this ");
        writer.println("#                package  directory into your CLASSPATH.");
        writer.println("#                Otherwise, system complains \"Can't find the filter class.\"");
        writer.println("#      SHORTKEY - filter shortkey keymap ");
        writer.println("#########################################################################");
     
        writeToFile(writer, 1);
        writeToFile(writer, 2);
        writeToFile(writer, 4);
     
        writer.close();
     } catch (Exception e) {
        System.out.println("sv_menu file writing error.");
     }
  }

  private void writeToFile(PrintWriter writer, int dim) {
     Vector level0Vec = new Vector();
     Vector level1Vec = new Vector();
     Vector level2Vec = new Vector();
     Vector level3Vec = new Vector();

     switch(dim) {
        case 1:
           for (Enumeration e=get1dElements(); e.hasMoreElements();) {
              MenuInfo info = (MenuInfo)e.nextElement();

              if (info.level == 0) 
                 level0Vec.addElement(info);
              else if (info.level == 1) 
	         level1Vec.addElement(info);
              else if (info.level == 2) 
	         level2Vec.addElement(info);
              else if (info.level == 3) 
	         level3Vec.addElement(info);
            } 

            writer.println("\nbegin 1d");
            break;

        case 2:
           for (Enumeration e=get2dElements(); e.hasMoreElements();) {
              MenuInfo info = (MenuInfo)e.nextElement();

              if (info.level == 0) 
                 level0Vec.addElement(info);
              else if (info.level == 1) 
	         level1Vec.addElement(info);
              else if (info.level == 2) 
	         level2Vec.addElement(info);
              else if (info.level == 3) 
	         level3Vec.addElement(info);
            } 

            writer.println("\nbegin 2d");
            break;

        case 4:
           for (Enumeration e=get4dElements(); e.hasMoreElements();) {
              MenuInfo info = (MenuInfo)e.nextElement();

              if (info.level == 0) 
                 level0Vec.addElement(info);
              else if (info.level == 1) 
	         level1Vec.addElement(info);
              else if (info.level == 2) 
	         level2Vec.addElement(info);
              else if (info.level == 3) 
	         level3Vec.addElement(info);
            } 

            writer.println("\nbegin vector");
            break;
     }

     try {

        for (Enumeration e=level0Vec.elements(); e.hasMoreElements();) {
           MenuInfo lv0item = (MenuInfo)e.nextElement();

           if (!lv0item.isItem) {
              writer.println("<menu> " + lv0item.name);

              for (Enumeration e1=level1Vec.elements(); e1.hasMoreElements();) {
                 MenuInfo lv1item = (MenuInfo)e1.nextElement();
              
                 if (lv1item.parent == lv0item.id) {
                    if (!lv1item.isItem) {
                       writer.println("   <menu> " + lv1item.name);
                       
                       for (Enumeration e2=level2Vec.elements(); e2.hasMoreElements();) {
                          MenuInfo lv2item = (MenuInfo)e2.nextElement();

                          if (lv2item.parent == lv1item.id) {
                             if (!lv2item.isItem) {
                                writer.println("      <menu> " + lv2item.name);

                                for (Enumeration e3=level3Vec.elements(); e3.hasMoreElements();) {
                                   MenuInfo lv3item = (MenuInfo)e3.nextElement();
                                   if (lv3item.parent == lv2item.id) {
                                      if (lv3item.isItem) {
                                         if (lv3item.shortkey > 0) 
                                            writer.println("        <item> NAME=" + "\"" + lv3item.name +
                                               "\", CLASS=" + lv3item.className +
                                               "\", PACKAGE=" + lv3item.directory + 
                                               "\", SHORTKEY=" + lv3item.shortkey);
                                         else
                                            writer.println("        <item> NAME=" + "\"" + lv3item.name +
                                               "\", CLASS=" + lv3item.className +
                                               "\", PACKAGE=" + lv3item.directory + "\"");
                                      }
                                   }
			        }

                                writer.println("      </menu>");
                             } else {
                                if (lv2item.shortkey > 0) 
                                    writer.println("     <item> NAME=\"" + lv2item.name +
                                          "\", CLASS=\"" + lv2item.className +
                                          "\", PACKAGE=\"" + lv2item.directory + 
                                          "\", SHORTKEY=" + lv2item.shortkey);
                                 else
                                    writer.println("        <item> NAME=\"" + lv2item.name +
                                          "\", CLASS=\"" + lv2item.className +
                                         "\", PACKAGE=\"" + lv2item.directory + "\"");
                             }
                          }
                       }
 
                       writer.println("   </menu>");
                    } else {
                       if (lv1item.shortkey > 0) 
                           writer.println("     <item> NAME=\"" + lv1item.name +
                                  "\", CLASS=\"" + lv1item.className +
                                  "\", PACKAGE=\"" + lv1item.directory +
                                  "\", SHORTKEY=" + lv1item.shortkey);
                       else
                           writer.println("     <item> NAME=\"" + lv1item.name +
                                  "\", CLASS=\"" + lv1item.className +
                                  "\", PACKAGE=\"" + lv1item.directory + "\"");
                    }
                 }
              }
              writer.println("</menu>");
           } else {
	      if (lv0item.shortkey > 0) 
                  writer.println("  <item> NAME=\"" + lv0item.name +
                         "\", CLASS=\"" + lv0item.className +
                         "\", PACKAGE=\"" + lv0item.directory +
                         "\", SHORTKEY=" + lv0item.shortkey);
               else
                  writer.println("  <item> NAME=\"" + lv0item.name +
                          "\", CLASS=\"" + lv0item.className +
                          "\", PACKAGE=\"" + lv0item.directory + "\"");
           }
        }
        writer.println("end\n");
     } catch (Exception e) {
        e.printStackTrace();
     }
  }


  // Each line should start with begin, <menu>, <item>, </item>, or end. 
  // It allow only 3 level total.
  private void parseMenuInfo(String sv_menu) {
     
     BufferedReader in = null;

     String line, tmp, menuName, token;
     int level = 0, index = 0, root0 = 0, root1 = 0, root2 = 0;

     String mname = null, mclass = null, mpackage = null;
     int mkey = -1;

     try {
        in = new BufferedReader(new FileReader(sv_menu));
     } catch (Exception e) {
        System.out.println("Can't find menu information file. System exits...");
        System.exit(1);
     }

     try {
        while (true) {
          String readed = in.readLine().toLowerCase();
          if ((readed.indexOf("begin") >= 0) && (readed.indexOf("1d") >= 0))
             break;
        }
 
        while (true) {
          line = in.readLine().trim();
                    
          if (line.equalsIgnoreCase("end")) 
             break;
 
          tmp = line.toLowerCase();
          if (tmp.indexOf("<menu>") >= 0) {
             menuName = line.substring(6);
             menuName = menuName.trim();
             index++;

             if (level == 0) {
                root0 = index;
                menuVec.addElement(new MenuInfo(menuName, level, index, -1));
             } else if (level == 1) {
                root1 = index;
                menuVec.addElement(new MenuInfo(menuName, level, index, root0));
             } else {
                root2 = index;
                menuVec.addElement(new MenuInfo(menuName, level, index, root1));
             }

             level++;
          } else if (tmp.indexOf("</menu>") >= 0) {
             level--;
          } else if (tmp.indexOf("<item>") >= 0) {
             index++;
             menuName = line.substring(6);

             mkey = -1;
             StringTokenizer st = new StringTokenizer(menuName, ",");
             while(st.hasMoreTokens()) {
                tmp = st.nextToken().trim();
                                
                int pos = tmp.indexOf("=");
                if (tmp.indexOf("NAME") >= 0) {
                   mname = tmp.substring(pos+2, tmp.length()-1);
                } else if (tmp.indexOf("CLASS") >= 0) {
                   mclass = tmp.substring(pos+2, tmp.length()-1);
                } else if (tmp.indexOf("PACKAGE") >= 0) {
                   mpackage = tmp.substring(pos+2, tmp.length()-1);
                } else if (tmp.indexOf("SHORTKEY") >= 0) {
                   mkey = Integer.valueOf(tmp.substring(pos+1, tmp.length())).intValue();
                }
             }

             if (level == 0) {
               menuVec.addElement(new MenuInfo(mname, mclass, mpackage,
                                       -1, level, index, -1));
             } else if (level == 1) {
               menuVec.addElement(new MenuInfo(mname, mclass, mpackage,
                                       mkey, level, index, root0));
             } else if (level == 2) {
               menuVec.addElement(new MenuInfo(mname, mclass, mpackage, 
                                       mkey, level, index, root1));
             } else {
               menuVec.addElement(new MenuInfo(mname, mclass, mpackage, 
                                       mkey, level, index, root2));
             }
          } 

	}
     } catch (Exception e) {
        System.out.println("Corrupted menu information(1D), system exits ...");
        e.printStackTrace();
        System.exit(1);
     }

     ////////////////////////////////////////////////
     // parse 2d menu information
     ////////////////////////////////////////////////
     try {

        while (true) {
          String readed = in.readLine().toLowerCase();
          if ((readed.indexOf("begin") >= 0) && (readed.indexOf("2d") >= 0))
             break;
        }
 
        while (true) {
          line = in.readLine().trim();
                    
          if (line.equalsIgnoreCase("end")) 
             break;
 
          tmp = line.toLowerCase();
          if (tmp.indexOf("<menu>") >= 0) {
             menuName = line.substring(6);
             menuName = menuName.trim();
             index++;

             if (level == 0) {
                root0 = index;
                menuVec2D.addElement(new MenuInfo(menuName, level, index, -1));
             } else if (level == 1) {
                root1 = index;
                menuVec2D.addElement(new MenuInfo(menuName, level, index, root0));
             } else {
                root2 = index;
                menuVec2D.addElement(new MenuInfo(menuName, level, index, root1));
             }

             level++;
          } else if (tmp.indexOf("</menu>") >= 0) {
             level--;
          } else if (tmp.indexOf("<item>") >= 0) {
             index++;
             menuName = line.substring(6);

             mkey = -1;
             StringTokenizer st = new StringTokenizer(menuName, ",");
             while(st.hasMoreTokens()) {
                tmp = st.nextToken().trim();
                                
                int pos = tmp.indexOf("=");
                if (tmp.indexOf("NAME") >= 0) {
                   mname = tmp.substring(pos+2, tmp.length()-1);
                } else if (tmp.indexOf("CLASS") >= 0) {
                   mclass = tmp.substring(pos+2, tmp.length()-1);
                } else if (tmp.indexOf("PACKAGE") >= 0) {
                   mpackage = tmp.substring(pos+2, tmp.length()-1);
                } else if (tmp.indexOf("SHORTKEY") >= 0) {
                   mkey = Integer.valueOf(tmp.substring(pos+1, tmp.length())).intValue();
                }
             }

             if (level == 0) {
               menuVec2D.addElement(new MenuInfo(mname, mclass, mpackage,
                                        -1, level, index, -1));
             } else if (level == 1) {
               menuVec2D.addElement(new MenuInfo(mname, mclass, mpackage,
                                        mkey, level, index, root0));
             } else if (level == 2) {
               menuVec2D.addElement(new MenuInfo(mname, mclass, mpackage, 
                                        mkey, level, index, root1));
             } else {
               menuVec2D.addElement(new MenuInfo(mname, mclass, mpackage, 
                                        mkey, level, index, root2));
             }
          } 

	}

     } catch (Exception e) {
        System.out.println("Corrupted menu information(2D), system exits ...");
        e.printStackTrace();
        System.exit(1);
     }

     ////////////////////////////////////////////////
     // parse 4d menu information
     ////////////////////////////////////////////////
     try {

        while (true) {
          String readed = in.readLine().toLowerCase();
          if ((readed.indexOf("begin") >= 0) && (readed.indexOf("vector") >= 0))
             break;
        }
 
        while (true) {
          line = in.readLine().trim();
                    
          if (line.equalsIgnoreCase("end")) 
             break;
 
          tmp = line.toLowerCase();
          if (tmp.indexOf("<menu>") >= 0) {
             menuName = line.substring(6);
             menuName = menuName.trim();
             index++;

             if (level == 0) {
                root0 = index;
                menuVec4D.addElement(new MenuInfo(menuName, level, index, -1));
             } else if (level == 1) {
                root1 = index;
                menuVec4D.addElement(new MenuInfo(menuName, level, index, root0));
             } else {
                root2 = index;
                menuVec4D.addElement(new MenuInfo(menuName, level, index, root1));
             }

             level++;
          } else if (tmp.indexOf("</menu>") >= 0) {
             level--;
          } else if (tmp.indexOf("<item>") >= 0) {
             index++;
             menuName = line.substring(6);

             mkey = -1;
             StringTokenizer st = new StringTokenizer(menuName, ",");
             while(st.hasMoreTokens()) {
                tmp = st.nextToken().trim();
                                
                int pos = tmp.indexOf("=");
                if (tmp.indexOf("NAME") >= 0) {
                   mname = tmp.substring(pos+2, tmp.length()-1);
                } else if (tmp.indexOf("CLASS") >= 0) {
                   mclass = tmp.substring(pos+2, tmp.length()-1);
                } else if (tmp.indexOf("PACKAGE") >= 0) {
                   mpackage = tmp.substring(pos+2, tmp.length()-1);
                } else if (tmp.indexOf("SHORTKEY") >= 0) {
                   mkey = Integer.valueOf(tmp.substring(pos+1, tmp.length())).intValue();
                }
             }

             if (level == 0) {
               menuVec4D.addElement(new MenuInfo(mname, mclass, mpackage,
                                        -1, level, index, -1));
             } else if (level == 1) {
               menuVec4D.addElement(new MenuInfo(mname, mclass, mpackage,
                                        mkey, level, index, root0));
             } else if (level == 2) {
               menuVec4D.addElement(new MenuInfo(mname, mclass, mpackage, 
                                        mkey, level, index, root1));
             } else {
               menuVec4D.addElement(new MenuInfo(mname, mclass, mpackage, 
                                        mkey, level, index, root2));
             }
          } 

	}

     } catch (Exception e) {
        System.out.println("Corrupted menu information(Vector), system exits ...");
        e.printStackTrace();
        System.exit(1);
     }

  }

} // end of class
