package sv.kernel;

import java.io.*;

/** 
 * SerialMgr.java
 * Management of file reading and writing and sending data over socket
 * @version 1.0   10/15/97
 * @author Byeongseob Ki
 * 
 */

public class SerialMgr {

  /** 
   * parse 1d data stream and return result time data
   */
  public static TimeData1D parse1dStream(DataInputStream in) {
    try {
       boolean COLOR_CODE = false;
       boolean ERRORS = false;
       boolean BB = false;
   
       float[] xArr = null, yArr = null, cArr = null, eArr= null;
       float xmin=0f, xmax=0f;

       // received time
       float time = in.readFloat();

       // number of points in given time and make arrays
       int noOfPoints = in.readInt();

       // received cArr
       if (in.readByte() == 1) {
          COLOR_CODE = true;
          cArr = new float[noOfPoints];
          for (int i=0; i<noOfPoints; i++) 
             cArr[i] = in.readFloat();
       } 

       // received error array
       if (in.readInt() == 1) {
          ERRORS = true;
          eArr = new float[noOfPoints];
          for (int i=0; i<noOfPoints; i++) 
             eArr[i] = in.readFloat();
       } 

       // receive reserved 1 item and flush out
       in.readInt();
       
       // received x and y arrays
       if (in.readByte() == 1) {
          BB = true;
          yArr = new float[noOfPoints];
        
          xmin = in.readFloat();
          xmax = in.readFloat();

          for (int i=0; i<noOfPoints; i++) 
             yArr[i] = in.readFloat();

       } else {
          xArr = new float[noOfPoints];
          yArr = new float[noOfPoints];

          for (int i=0; i<noOfPoints; i++) {
             xArr[i] = in.readFloat();
             yArr[i] = in.readFloat();
          }
       }

       // SHOULD BE IMPLEMENTED FOR VECTOR_INCLUDED
       if (BB) {
         if (COLOR_CODE && ERRORS) 
           return null;   // not implemented yet
         else if (COLOR_CODE && !ERRORS)
           return null;   // not implemented yet
         else if (!COLOR_CODE && ERRORS)
           return new TimeData1D(time, noOfPoints, xmin, xmax, yArr, eArr);
         else  
           return new TimeData1D(time, noOfPoints, xmin, xmax, yArr);

       } else {

         if (COLOR_CODE && ERRORS) 
           return new TimeData1D(time, noOfPoints, xArr, yArr, eArr, cArr); 
         else if (COLOR_CODE && !ERRORS)
           return new TimeData1D(time, noOfPoints, xArr, yArr, cArr); 
         else if (!COLOR_CODE && ERRORS)
           return new TimeData1D(time, noOfPoints, xArr, yArr, eArr, true);
         else 
           return new TimeData1D(time, noOfPoints, xArr, yArr);
       }

    } catch (Exception E) {
       System.out.println("Error while reading data(1d) ...");
       return null;
    }
  }


  /** 
   * parse 2d data stream and return result time data
   */
  public static TimeData2D parse2dStream(DataInputStream in) {
    boolean COLOR_CODE = false;
    boolean VECTOR_INCLUDED = false;     // not used yet
    boolean BB = false;
   
    TimeDataVec vectorSet = null;
    float[] xArr = null, yArr = null, zArr = null, cArr = null;
    float xmin=0f, ymin=0f, xmax=0f, ymax=0f;

    try {
       
       // received time
       float time = in.readFloat();

       // number of points in given time and make arrays
       int nx = in.readInt();
       int ny = in.readInt();
       int noOfPoints = nx * ny;

       // received cArr
       if (in.readByte() == 1) {
          COLOR_CODE = true;
          cArr = new float[noOfPoints];
          for (int i=0; i<noOfPoints; i++) 
             cArr[i] = in.readFloat();
       } 

       // received vector set
       if (in.readByte() == 1) {
          vectorSet = parsevecStream(in);
          VECTOR_INCLUDED = true;
       }

       // receive reserved 2 items and flush out
       in.readInt();
       in.readInt();       

       // received x, y, and z arrays
       if (in.readByte() == 1) {
          BB = true;
          zArr = new float[noOfPoints];
        
          xmin = in.readFloat();
          xmax = in.readFloat();
          ymin = in.readFloat();
          ymax = in.readFloat();

          for (int i=0; i<noOfPoints; i++) 
             zArr[i] = in.readFloat();

       } else {
          xArr = new float[noOfPoints];
          yArr = new float[noOfPoints];
          zArr = new float[noOfPoints];

          for (int i=0; i<noOfPoints; i++) {
             xArr[i] = in.readFloat();
             yArr[i] = in.readFloat();
             zArr[i] = in.readFloat();
          }
       }

       // SHOULD BE IMPLEMENTED FOR VECTOR_INCLUDED
       if (BB) {
	 if (COLOR_CODE) 
           return new TimeData2D(time, xmin, xmax, nx, 
                                      ymin, ymax, ny, zArr, cArr);
         else 
           return new TimeData2D(time, xmin, xmax, nx, 
                                    ymin, ymax, ny, zArr);

       } else {
          // SHOULD BE IMPLEMENTED FOR COLOR_CODE
          return new TimeData2D(time, noOfPoints, xArr, yArr, zArr);
       }


    } catch (Exception E) {
       System.out.println("Error while reading data(2d) ...");
       return null;
    }
  }

  /** 
   * parse 3d data stream and return result time data
   */
  public static TimeData3D parse3dStream(DataInputStream in) {

    try {
       
       // received time
       float time = in.readFloat();

       int dimx = in.readInt();  
       int dimy = in.readInt();  
       int dimz = in.readInt();  

       float threshold = in.readFloat();
       float zmin = in.readFloat();
       float zmax = in.readFloat();

       // receive reserved 2 items and flush out
       in.readInt();
       in.readInt();       

       int vsize = in.readInt();     // read vertex size
       float[][] vertex = new float[vsize][3];
       float[][] normal = new float[vsize][3];

       for (int i=0; i<vsize; i++) { 
	 for (int j=0; j<3; j++) 
             vertex[i][j] = in.readFloat();
         for (int j=0; j<3; j++) 
             normal[i][j] = in.readFloat(); 
       }

       int tsize = in.readInt();     // read triangle
       int[][] triangle = new int[tsize][3];

       for (int i=0; i<tsize; i++) 
          for (int j=0; j<3; j++) 
             triangle[i][j] = in.readInt();


       return  new TimeData3D(time, vsize, vertex, normal, tsize, triangle,
                             dimx, dimy, dimz, threshold, zmin, zmax);

    } catch (Exception E) {
       System.out.println("Error while reading data(3d) ...");
E.printStackTrace();
       return null;
    }
  }


  /** 
   * parse connection list data stream and return result time data
   */
  public static TimeDataCon parseconStream(DataInputStream in) {
    try {
       boolean BB = false;
   
       float[] cArr = null, xArr = null, yArr = null;
       float xmin=0f, xmax=0f;

       // received time
       float time = in.readFloat();

       // number of points in given time and make arrays
       int noOfPoints = in.readInt();

       // receive reserved 2 items and flush out
       in.readInt();
       in.readInt();      

       // received x, y, r and t arrays
       if (in.readByte() == 1) {
          BB = true;
          yArr = new float[noOfPoints];
          cArr = new float[noOfPoints];

          xmin = in.readFloat();
          xmax = in.readFloat();

          for (int i=0; i<noOfPoints; i++) {
             yArr[i] = in.readFloat();
             cArr[i] = in.readFloat();
          }

       } else {
          xArr = new float[noOfPoints];
          yArr = new float[noOfPoints];
          cArr = new float[noOfPoints];

          for (int i=0; i<noOfPoints; i++) {
             xArr[i] = in.readFloat();
             yArr[i] = in.readFloat();
             cArr[i] = in.readFloat();
          }
       }

       // send number of connection points in given time
       int clsz = in.readInt();
       int[] connection = new int[clsz];

       for (int i=0; i<clsz; i++) 
         connection[i] = in.readInt();



       if (BB) {
         return null;   // not implemented yet

       } else {
         return new TimeDataCon(time, xArr, yArr, cArr, connection);
       }

    } catch (Exception E) {
       System.out.println("Error while reading data(connection list) ...");
       return null;
    }
  }


  /** 
   * parse vector data stream and return result time data
   */
  public static TimeDataVec parsevecStream(DataInputStream in) {
    try {
       boolean COLOR_CODE = false;
       boolean BB = false;
   
       float[] cArr = null;
       float[] xArr = null, yArr = null, rArr = null, tArr = null;
       float xmin=0f, xmax=0f;

       // received time
       float time = in.readFloat();

       // number of points in given time and make arrays
       int noOfPoints = in.readInt();


       // received cArr
       if (in.readByte() == 1) {
          COLOR_CODE = true;
          cArr = new float[noOfPoints];
          for (int i=0; i<noOfPoints; i++) 
             cArr[i] = in.readFloat();
       } 

       // receive reserved 2 items and flush out
       in.readInt();
       in.readInt();      

       // received x, y, r and t arrays
       if (in.readByte() == 1) {
          BB = true;
          yArr = new float[noOfPoints];
          rArr = new float[noOfPoints];
          tArr = new float[noOfPoints];

          xmin = in.readFloat();
          xmax = in.readFloat();

          for (int i=0; i<noOfPoints; i++) {
             yArr[i] = in.readFloat();
             rArr[i] = in.readFloat();
             tArr[i] = in.readFloat();
          }

       } else {
          xArr = new float[noOfPoints];
          yArr = new float[noOfPoints];
          rArr = new float[noOfPoints];
          tArr = new float[noOfPoints];

          for (int i=0; i<noOfPoints; i++) {
             xArr[i] = in.readFloat();
             yArr[i] = in.readFloat();
             rArr[i] = in.readFloat();
             tArr[i] = in.readFloat();
          }
       }

       if (BB) {
         if (COLOR_CODE) 
           return null;   // not implemented yet
         else 
           return null;   // not implemented yet

       } else {
         if (COLOR_CODE) 
           return null;   // not implemented yet
         else 
           return new TimeDataVec(time, xArr, yArr, rArr, tArr);
      }

    } catch (Exception E) {
       System.out.println("Error while reading data(vector) ...");
       return null;
    }
  }


  /** 
   * parse bar chart data stream and return result time data
   */
  public static TimeDataBar parsebarStream(DataInputStream in) {
    try {
       float[] xArr = null, yArr = null, wArr = null, cArr = null;

       // received time
       float time = in.readFloat();

       // number of points in given time and make arrays
       int noOfPoints = in.readInt();

       // received cArr
       if (in.readByte() == 1) {
          cArr = new float[noOfPoints];
          for (int i=0; i<noOfPoints; i++) 
             cArr[i] = in.readFloat();
       } else {
System.out.println("Non color bar chart data is not supported ...");
return null; 
       } 

       // receive reserved 2 items and flush out
       in.readInt();
       in.readInt();      

       // received x, y, and w arrays
       if (in.readByte() == 1) {
System.out.println("Bounding box of bar chart data is not supported ...");
return null; 
       } else {
          xArr = new float[noOfPoints];
          yArr = new float[noOfPoints];
          wArr = new float[noOfPoints];

          for (int i=0; i<noOfPoints; i++) {
             xArr[i] = in.readFloat();
             yArr[i] = in.readFloat();
             wArr[i] = in.readFloat();
          }
       }

       return new TimeDataBar(time, xArr, yArr, wArr, cArr);

    } catch (Exception E) {
       System.out.println("Error while reading data(bar chart) ...");
       return null;
    }
  }


  /** 
   * parse tree data stream and return result time data
   */
  public static TimeDataTree parsetreeStream(DataInputStream in) {
    try {
       float[] xArr = null, yArr = null, xtoArr = null, ytoArr = null;

       // received time
       float time = in.readFloat();

       // number of points in given time and make arrays
       int noOfPoints = in.readInt();

       // receive reserved 3 items and flush out
       in.readByte();
       in.readInt();
       in.readInt();      

       // received x, y, xto and yto arrays
       if (in.readByte() == 1) {
System.out.println("Bounding box of tree data is not supported ...");
return null; 
       } else {
          xArr = new float[noOfPoints];
          yArr = new float[noOfPoints];
          xtoArr = new float[noOfPoints];
          ytoArr = new float[noOfPoints];

          for (int i=0; i<noOfPoints; i++) {
             xArr[i] = in.readFloat();
             yArr[i] = in.readFloat();
             xtoArr[i] = in.readFloat();
             ytoArr[i] = in.readFloat();
          }
       }

       return new TimeDataTree(time, xArr, yArr, xtoArr, ytoArr);

    } catch (Exception E) {
       System.out.println("Error while reading data(tree data) ...");
       return null;
    }
  }


}  // end of class

