package sv.kernel;

import java.io.*;

/** 
 * TimeDataBar.java
 * Define vector set for vector plot
 * @version 1.0   5/1/98
 * @author Byeongseob Ki
 */

public class TimeDataBar extends TDPlane {

  private float[] wArr;     // width vals

  public TimeDataBar(float time, float[] xArr, float[] yArr,
                                          float[] wArr, float[] cArr) {
     super(time, xArr.length, xArr, yArr);
     dataType = DSPlane.BAR;

     this.wArr = wArr;
     this.cArr = cArr;
     recalcMinMax();
  }

  /** 
   * sending data by using DataOutputStream
   */
  public void sendingData(DataOutputStream out) {

    try {
       // send time
       out.writeFloat(time);

       // send number of points in given time
       out.writeInt(noOfPoints);
     
       // send cArr
       if (cArr == null) {
          out.writeByte(0);
       } else {
          out.writeByte(1);
          for (int i=0; i<noOfPoints; i++) 
             out.writeFloat(cArr[i]);
       }

       // send reserved for future use (2 items)
       out.writeInt(0);
       out.writeInt(0);

       // send x and y arrays
       if (bb) {
         out.writeByte(1);     // bb is true... send boundbox first
         out.writeFloat(xmin);
         out.writeFloat(xmax);

         for (int i=0; i<noOfPoints; i++) {
           out.writeFloat(yArr[i]);
           out.writeFloat(wArr[i]);
         }

       } else {
         out.writeByte(0);      // bb is false

         for (int i=0; i<noOfPoints; i++) {
            out.writeFloat(xArr[i]);
            out.writeFloat(yArr[i]);
            out.writeFloat(wArr[i]);
         }
       }

    } catch (Exception e) {
       System.out.println("Data sending error(TimeDataBar).");
       //e.printStackTrace();
    } 
  }


  /**
   * update data 
   * @param data TimeDataBar to be appended or updated
   */
  public void updateData(TimeDataBar data, boolean parallel) {
     System.out.println("Update time data function is not implemented.");
  }


  /**
   * return wArr
   */
  public float[] getwArr() {
     int sz = wArr.length;
     float[] ret = new float[sz];
     System.arraycopy(wArr, 0, ret, 0, sz);

     return ret;
  }

  private void recalcMinMax() {
     // check first value (is it centered?)
     if (cArr[0] < 1000f) 
        xmin = xmin - wArr[0]/2.0f;

     // check first value (is it left aligned?)
     if (cArr[0] >= 2000f) 
        xmin = xmin - wArr[0];

     // check last value (is it centered?)
     if (cArr[cArr.length-1] < 1000f) 
        xmax = xmax + wArr[0]/2.0f;

     // check last value (is it right aligned?)
     if (cArr[cArr.length-1] >= 1000f && cArr[cArr.length-1] < 2000f) 
        xmax = xmax + wArr[0];
  }

} // end of class

