package sv.kernel;

import java.io.*;

/** 
 * TimeDataTree.java
 * Define tree data set 
 * @version 1.0   5/1/98
 * @author Byeongseob Ki
 */

public class TimeDataTree extends TDPlane {

  private float[] xtoArr;     // x to array
  private float[] ytoArr;     // y to array

  public TimeDataTree(float time, float[] xfromArr, float[] yfromArr,
                              float[] xtoArr, float[] ytoArr) {
     super(time, xfromArr.length, xfromArr, yfromArr);
     dataType = DSPlane.TREE;

     this.xtoArr = xtoArr;
     this.ytoArr = ytoArr;
  }

  public void addMorePoints(TimeDataTree prevData) {
     recalcMinMax(prevData);
  }


  /** 
   * sending data by using DataOutputStream
   */
  public void sendingData(DataOutputStream out) {

    try {
       // send time
       out.writeFloat(time);

       // send number of points in given time
       out.writeInt(noOfPoints);
     
       // send reserved for future use (3 items)
       out.writeByte(0);
       out.writeInt(0);
       out.writeInt(0);

       // send x and y arrays
       if (bb) {
         System.out.println("Undefined for bounding box ...");
         out.writeByte(1);
       } else {
         out.writeByte(0);      // bb is false

         for (int i=0; i<noOfPoints; i++) {
            out.writeFloat(xArr[i]);
            out.writeFloat(yArr[i]);
            out.writeFloat(xtoArr[i]);
            out.writeFloat(ytoArr[i]);
         }
       }

    } catch (Exception e) {
       System.out.println("Data sending error(TimeDataTree).");
       //e.printStackTrace();
    } 
  }


  /**
   * update data 
   * @param data TimeDataTree to be appended or updated
   */
  public void updateData(TimeDataTree data, boolean parallel) {
     System.out.println("Update time data function is not implemented.");
  }

  /**
   * return xtoArr
   */
  public float[] getxtoArr() {
     int sz = xtoArr.length;
     float[] ret = new float[sz];
     System.arraycopy(xtoArr, 0, ret, 0, sz);

     return ret;
  }


  /**
   * return ytoArr
   */
  public float[] getytoArr() {
     int sz = ytoArr.length;
     float[] ret = new float[sz];
     System.arraycopy(ytoArr, 0, ret, 0, sz);

     return ret;
  }

  private void recalcMinMax(TimeDataTree prevData) {
     if (prevData == null) return;
     
     float[] prevxArr = prevData.getxArr();
     float[] prevyArr = prevData.getyArr();
     float[] prevxtoArr = prevData.getxtoArr();
     float[] prevytoArr = prevData.getytoArr();

     float[] xfromArr = getxArr();
     float[] yfromArr = getyArr();
     float[] xxtoArr = getxtoArr();
     float[] yytoArr = getytoArr();
     
     noOfPoints = xfromArr.length + prevxArr.length;
     
     xArr = new float[noOfPoints];
     yArr = new float[noOfPoints];
     xtoArr = new float[noOfPoints];
     ytoArr = new float[noOfPoints];

     System.arraycopy(prevxArr, 0, xArr, 0, prevxArr.length);
     System.arraycopy(xfromArr, 0, xArr, prevxArr.length, xfromArr.length);
     System.arraycopy(prevyArr, 0, yArr, 0, prevxArr.length);
     System.arraycopy(yfromArr, 0, yArr, prevxArr.length, xfromArr.length);
     System.arraycopy(prevxtoArr, 0, xtoArr, 0, prevxArr.length);
     System.arraycopy(xxtoArr, 0, xtoArr, prevxArr.length, xfromArr.length);
     System.arraycopy(prevytoArr, 0, ytoArr, 0, prevxArr.length);
     System.arraycopy(yytoArr, 0, ytoArr, prevxArr.length, xfromArr.length);

     for (int i=0; i< noOfPoints; i++) {
        if (xmin > xArr[i]) xmin = xArr[i];
        if (xmax < xArr[i]) xmax = xArr[i];
        if (ymin > yArr[i]) ymin = yArr[i];
        if (ymax < yArr[i]) ymax = yArr[i];
     }
  }

} // end of class

