package sv.kernel;

import java.awt.*;
import java.util.Vector;

public class TriObject {

    float vert[];
    int tvert[];

    int nvert, maxvert, triLimit = Integer.MAX_VALUE, vertLimit;

    /// polygon info ////////////
    int tri1[], tri2[], tri3[], tri4[];

    float triDepth[]; 
    TColor triColor[];
    public Vector colorIndex = new Vector();

    int ntri, maxtri;
    public boolean transformed;

    Matrix3D mat;

    public float xmin, xmax, ymin, ymax, zmin, zmax;

    /// Global display parameters
    public boolean fillSurface = false;
    public boolean wireSurface = true;
    public boolean doShading = false;

    public float xcam = 0.0f;
    public float ycam = 0.0f;
    public float zcam = 500.0f;

    /**
     * constructor
     */
    public TriObject() {
        mat = new Matrix3D ();
        mat.xrot(20);
        mat.yrot(30);
    }

    /**
     * set the triangle limit for the bounding box and surface grids
     */
    public void setTriangleLimits(int lim1, int lim2) {
       triLimit = lim1;
       vertLimit = lim2;
    }

    /**
     * set fill mode
     */
    public void setFillMode(boolean b) {
        int indexSize = colorIndex.size();
        int c1, c2;
        Color colors[] = { new Color(0, 0, 255), 
                           new Color(0, 255, 0),
                           new Color(255, 0, 0) };
// allow 3 data sets
	/*
        switch (indexSize) {
           case 1:
              c1 = ((Integer)colorIndex.elementAt(0)).intValue();
              for (int i=0; i<c1; i++) 
                 triColor[i] = colors[0];
              for (int i=c1; i<ntri; i++) 
                 triColor[i] = colors[1];
              break;
           case 2:
              c1 = ((Integer)colorIndex.elementAt(0)).intValue();
              c2 = ((Integer)colorIndex.elementAt(0)).intValue();
              for (int i=0; i<c1; i++) 
                 triColor[i] = colors[0];
              for (int i=c1; i<c2; i++) 
                 triColor[i] = colors[0];
              for (int i=c2; i<ntri; i++) 
                 triColor[i] = colors[1];
              break;
           default:
              for (int i=0; i<ntri; i++) 
                 triColor[i] = colors[0];
              break;
        }
	*/

        for (int i=0; i<ntri; i++) {
           if (triColor[i].flag == 0)
              triColor[i].color = colors[0];
        }
        fillSurface = b;
    }

    /**
     * set wire mode
     */
    public void setWireMode(boolean b) {
        wireSurface = b;
    }

    /**
     * set shading mode
     */
    public void setShadingMode(boolean b) {
        doShading = b;
    }

    /**
     * Transform all the points in this model. 
     */
    void transform() {
        if (transformed || nvert <= 0) 
            return;

        if (tvert == null || tvert.length < nvert * 3)
            tvert = new int[nvert*3];
        mat.transform(vert, tvert, nvert);

        transformed = true;
    }

    /**
     * Add a point to the vertex list for this TriObject.
     */
    public int addVert(float x, float y, float z) {
        int i = nvert;
        if (i >= maxvert)
            if (vert == null) {
                maxvert = 100;
                vert = new float[maxvert * 3];
            } else {
                maxvert *= 2;
                float nv[] = new float[maxvert * 3];
                System.arraycopy(vert, 0, nv, 0, vert.length);
                vert = nv;
            }
        i *= 3;
        vert[i] = x;
        vert[i + 1] = y;
        vert[i + 2] = z;

        return nvert++;
    }

   /**
    * Add a line from vertex p1 to vertex p2.
    */
   public void add(int p1, int p2, Color color) {

        int i = ntri;
        if (p1 >= nvert || p2 >= nvert)
            return;
        if (i >= maxtri)
            if (tri1 == null) {
                maxtri = 100;
                tri1 = new int[maxtri];
                tri2 = new int[maxtri];
                tri3 = new int[maxtri];
                tri4 = new int[maxtri];
		triColor = new TColor[maxtri];
            } else {
                maxtri *= 2;
                int nv1[] = new int[maxtri];
                int nv2[] = new int[maxtri];
                int nv3[] = new int[maxtri];
                int nv4[] = new int[maxtri];
                TColor col[] = new TColor[maxtri];

                System.arraycopy(tri1, 0, nv1, 0, tri1.length);
                System.arraycopy(tri2, 0, nv2, 0, tri2.length);
                System.arraycopy(tri3, 0, nv3, 0, tri3.length);
                System.arraycopy(tri4, 0, nv4, 0, tri4.length);
                System.arraycopy(triColor, 0, col, 0, triColor.length);

                tri1 = nv1;
                tri2 = nv2;
                tri3 = nv3;
                tri4 = nv4;
                triColor = col;
            }

        tri1[i] = p1;
        tri2[i] = p2;
        tri3[i] = -1;
        tri4[i] = -1;

        if (triLimit > i)
	   triColor[i] = new TColor(color, (byte)0);
        else
           triColor[i] = new TColor(color, (byte)1);

        ntri = i + 1;
    }

    /**
     * Add a polygon.
     */
    public void addPoly(int numv, int[] p, Color color) {

        int i = ntri;

        if (p[0] >= nvert || p[1] >= nvert || p[2] >= nvert)
            return;

        if (i >= maxtri)
            if (tri1 == null) {
                maxtri = 100;
		tri1 = new int[maxtri];
		tri2 = new int[maxtri];
		tri3 = new int[maxtri];
                tri4 = new int[maxtri];
		triDepth = new float[maxtri];
		triColor = new TColor[maxtri];
            } else {
                maxtri *= 2;
                int nv1[] = new int[maxtri];
                int nv2[] = new int[maxtri];
                int nv3[] = new int[maxtri];
                int nv4[] = new int[maxtri];
                float depth[] = new float[maxtri];
                TColor col[] = new TColor[maxtri];

                System.arraycopy(tri1, 0, nv1, 0, tri1.length);
                System.arraycopy(tri2, 0, nv2, 0, tri2.length);
                System.arraycopy(tri3, 0, nv3, 0, tri3.length);
                System.arraycopy(tri4, 0, nv4, 0, tri4.length);
                System.arraycopy(triDepth, 0, depth, 0, triDepth.length);
                System.arraycopy(triColor, 0, col, 0, triColor.length);

                tri1 = nv1;
                tri2 = nv2;
                tri3 = nv3;
                tri4 = nv4;
                triDepth = depth;
                triColor = col;
            }

	tri1[i] = p[0];
	tri2[i] = p[1];
	tri3[i] = p[2];
        if (numv > 3)
           tri4[i] = p[3];
	else 
           tri4[i] = -1;
	
       if (triLimit > i)
	   triColor[i] = new TColor(color, (byte)0);
        else
           triColor[i] = new TColor(color, (byte)1);

        ntri = i + 1;
    }

    /** 
     * draw objects
     */
    public void paint(Graphics g) {

       if (vert == null || triDepth == null) 
            return;

        int lim = ntri;

        if (lim <= 0 || nvert <= 0) 
            return;

        if (fillSurface || doShading) {
           Color tc = new Color(0, 0, 255);

           for (int i=0; i<ntri; i++) {
             if (triColor[i].flag == 0)
                triColor[i].color = tc;
           }
        }

        transform();
	depthSort();

        int c1[] = tri1;
        int c2[] = tri2;
	int c3[] = tri3;
        int c4[] = tri4;

        int v[] = tvert;

	int[] xpts = new int[4];
	int[] ypts = new int[4];
        int npts;

        for (int i = lim-1; i >= 0 ; i--) {
	    int p1 = c1[i]*3;
	    int p2 = c2[i]*3;
	    int p3 = c3[i]*3;
            int p4 = c4[i]*3;

	    if (p3 > 0) {                // triangle or rectangle
                if (p4 < 0)
                   npts = 3;
                else
                   npts = 4;

		xpts[0] = v[p1];
		xpts[1] = v[p2];
		xpts[2] = v[p3];
		ypts[0] = v[p1+1];
		ypts[1] = v[p2+1];
		ypts[2] = v[p3+1];

                if (npts > 3) {
                   xpts[3] = v[p4];
                   ypts[3] = v[p4+1];
                }

		if (fillSurface) {
                   g.setColor(triColor[i].color);
                   g.fillPolygon( xpts, ypts, npts ); 
		}
		if (wireSurface) {
                   g.setColor(Color.black);
                   g.drawPolygon( xpts, ypts, npts ); 
		}
	    } else  {                     // line
		g.setColor(triColor[i].color);
		g.drawLine(v[p1],v[p1+1],v[p2],v[p2+1]);
	    }
        }
    }


/***************************************************************************
 * depthSort
 *
 *	Sort the lines and polygons in the display list for distance 
 *	from the camera, so that they can be drawn in the correct order.
 *	The point is to make the ones in front overdraw the ones in back.
 *	Actually, depthSort only calculates the depths.  sortTriangles
 *	does the actual sorting.  depthSort also calculates the appropriate
 *	polygon color based on the angle of the surface normal with
 *	respect to the camera.
 *
 ***************************************************************************/
    void depthSort() {

	// Reality check
        int lim = ntri;
        if (lim <= 0 || nvert <= 0)
            return;

	// Calculate camera position
	float rcam = (float) Math.sqrt((double) 
			(xcam*xcam + ycam*ycam + zcam*zcam));

	float xcamhat = (float) (xcam/rcam);
	float ycamhat = (float) (ycam/rcam);	
        float zcamhat = (float) (zcam/rcam);

	// First calculate all the depths
        for (int i=0; i<lim; i++) {

	    int p1 = tri1[i]*3;
	    int p2 = tri2[i]*3;
	    int p3 = tri3[i]*3;

	    if (p3 > 0) {                    // if triangle or rectangle

	      /*
                float xmid = (vert[p1] + vert[p2] + vert[p3]) / 3.0f;
	        float ymid = (vert[p1+1] + vert[p2+1] + vert[p3+1]) / 3.0f;
		float zmid = (tvert[p1+2] + tvert[p2+2] + tvert[p3+2]) / 3.0f;

		float xdiff = xcam - xmid;
		float ydiff = ycam - ymid;
		float zdiff = zcam - zmid;

		triDepth[i] = (float)Math.sqrt((double)
			(xdiff*xdiff + ydiff*ydiff + zdiff*zdiff)); 

	      */

		float zmid = (tvert[p1+2] + tvert[p2+2] + tvert[p3+2]) / 3.0f;
		float zdiff = zcam - zmid;
                triDepth[i] =  zdiff*zdiff;


	    } else {	                    // if line
	      /*
		float xmid = (vert[p1] + vert[p2]) / 2.0f;
		float ymid = (vert[p1+1] + vert[p2+1]) / 2.0f;
		float zmid = (vert[p1+2] + vert[p2+2]) / 2.0f;

		float xdiff = xcam - xmid;
		float ydiff = ycam - ymid;
		float zdiff = zcam - zmid;

                triDepth[i] =  xdiff*xdiff + ydiff*ydiff + zdiff*zdiff;
	      */

		float zmid = (vert[p1+2] + vert[p2+2]) / 2.0f;
		float zdiff = zcam - zmid;
                triDepth[i] =  zdiff*zdiff;

	    }
        }

	// Then calculate all the colors (optional)
        for (int i=0; i<ntri && (doShading); i++) {
            if (triColor[i].flag != 0) continue;

	    int p1 = tri1[i]*3;
	    int p2 = tri2[i]*3;
	    int p3 = tri3[i]*3;

	    if (p3 > 0) {

		/* compute first X prod vector */
		float ax = vert[p1] - vert[p2]; 
		float ay = vert[p1+1] - vert[p2+1]; 
		float az = vert[p1+2] - vert[p2+2]; 

		/* compute sceond X prod vector */
		float bx = vert[p1] - vert[p3]; 
		float by = vert[p1+1] - vert[p3+1]; 
		float bz = vert[p1+2] - vert[p3+2]; 
		
		/* take the X product of a and b */
		float cx = (float) (ay*bz - az*by);
		float cy = (float) (az*bx - ax*bz);
		float cz = (float) (ax*by - ay*bx);

		/* normalize the normal */
		float rnorm = (float) Math.sqrt((double)
					(cx*cx + cy*cy + cz*cz));

		float cxhat, cyhat, czhat;
		if (rnorm != 0.0f) {
			cxhat = (float) (cx/rnorm);
			cyhat = (float) (cy/rnorm);
			czhat = (float) (cz/rnorm);
		} else {
			cxhat = 1.0f;
			cyhat = 1.0f;
			czhat = 1.0f;
		}

		/* projection of normal onto camera vector (cam dot norm) */
		float proj = (float)(xcamhat*cxhat + ycamhat*cyhat + zcamhat*czhat) * 1.2f;
		if (proj < 0)    proj =  -proj;
                if (proj > 1.0f) proj = 1.0f;

		//System.out.println(proj);

                int bc = triColor[i].color.getBlue();
                if (bc > 10) bc = (int)(bc * proj);
  		triColor[i].color = new Color(0, 0, bc);

	    }
	}

        if (fillSurface)
           sortTriangles(0,lim-1);
    }


   /** 
    * clear transformation matrix to unit matrix 
    */
   public void resetPosition () {
      mat.unit();
   }

   /** 
    * translate the object 
    */
   public void translate ( float x, float y, float z ) {
      mat.translate( x, y, z);
   }

   /** 
    * take into account the previous transformation 
    */
   public void relativePosition ( Matrix3D prevXForm ) {
      mat.mult(prevXForm);
   }

   /** 
    * scale the object 
    */
   public void scale ( float x, float y, float z ) {
      mat.scale(x, y, z);
   }

   /**
    * return the number of triangles
    */
   public int getNTRI() {
      return triLimit;
   }

   /**
    * return the number of vertex
    */
   public int getNVERT() {
      return vertLimit;
   }

    /** 
     * Find the bounding box of this model 
     */
    public void findBB() {

        if (nvert <= 0)
            return;
        float v[] = vert;
        float xmin = v[0], xmax = xmin;
        float ymin = v[1], ymax = ymin;
        float zmin = v[2], zmax = zmin;
        for (int i = nvert * 3; (i -= 3) > 0;) {
            float x = v[i];
            if (x < xmin)
                xmin = x;
            if (x > xmax)
                xmax = x;
            float y = v[i + 1];
            if (y < ymin)
                ymin = y;
            if (y > ymax)
                ymax = y;
            float z = v[i + 2];
            if (z < zmin)
                zmin = z;
            if (z > zmax)
                zmax = z;
        }
        this.xmax = xmax;
        this.xmin = xmin;
        this.ymax = ymax;
        this.ymin = ymin;
        this.zmax = zmax;
        this.zmin = zmin;
    }

    // The polygons have depths. They just need to be sorted.  This
    // sort is based on the version from Model3d.
    private void sortTriangles( int min, int max ) {

	int lo = min;
	int hi = max;
        if (lo >= hi)
            return;
	int midndx = (int) (lo+hi)/2;

        float mid = triDepth[midndx];

	int loopCount = 0;
        while (lo < hi) {
	    if (loopCount > 100000) {

		System.out.println ("Infinite loop in depthSort");
		break;
	    }
	    loopCount++;
            while (lo < hi && triDepth[lo] < mid) {
                lo++;
            }
            while (lo < hi && triDepth[hi] >= mid) {
                hi--;
            }

	    int T;
	    float D;
	    TColor C;
            if ((lo < hi) && (triDepth[lo] != triDepth[hi])) {
                T = tri1[lo];
                tri1[lo] = tri1[hi];
                tri1[hi] = T;

                T = tri2[lo];
                tri2[lo] = tri2[hi];
                tri2[hi] = T;

                T = tri3[lo];
                tri3[lo] = tri3[hi];
                tri3[hi] = T;

                T = tri4[lo];
                tri4[lo] = tri4[hi];
                tri4[hi] = T;

		D = triDepth[lo];
		triDepth[lo] = triDepth[hi];
		triDepth[hi] = D;

		C = triColor[lo];
		triColor[lo] = triColor[hi];
		triColor[hi] = C;

            }
        }
        if (hi < lo) {
            int T = hi;
            hi = lo;
            lo = T;
        }
        sortTriangles(min, lo);
        sortTriangles(lo == min ? lo + 1 : lo, max);
    }


   class TColor {
      Color color;
      byte flag = 0;         // if flag is 0, changable
 
      public TColor(Color c, byte b) {
         color = c;
         flag = b;
      }
   } // end of class TColor

} // end of class


