/* java_ser3d.c */

#include <stdio.h>
#include <stdlib.h>
#include "sockio.h"
#include "binio.h"


/*************************************************************
 * java_ser3d_
 * c program to link with fortran 90
 * This program writes the xsize, ysize, zsize and z values to socket.
 *
 * input parameters
 *   name - name of the file, it will be used for title
 *   time - time value
 *   nx    - x dimension 
 *   ny    - y dimension 
 *   nz    - z dimension 
 *   threshold - initial threshold
 *   z   - z array
 ************************************************************/

int java_ser3d_(name, time, nx, ny, nz, threshold, grid) 
  char *name;
  double *time;
  int *nx;
  int *ny;
  int *nz; 
  double *threshold;
  double grid[];
{
   int sockfd;
   int nnx = *nx, nny = *ny, nnz = *nz, name_length, i;
   int size = nnx * nny * nnz;
   double ft = *time, thre = *threshold;
   float thresh = (float)thre;
   float ftime = (float)ft;
   int command = 1;

   sockfd = initialize2();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);

   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* write time and sizes and threshold */
   write_int4(sockfd, 1);     /* send no of time */
   write_float4(sockfd, thresh);
   write_float4(sockfd, ftime);

   write_int4(sockfd, nnx);
   write_int4(sockfd, nny);
   write_int4(sockfd, nnz);

   /* write x and fx value */
   for (i=0; i<size; i++) 
      write_float4(sockfd, (float)grid[i]);
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_mser3d_
 * c program to link with fortran 90
 * This program writes the xsize, ysize, zsize and z values to socket.
 *
 * input parameters
 *   name - name of the file, it will be used for title
 *   time - time value
 *   nx    - x dimension 
 *   ny    - y dimension 
 *   nz    - z dimension 
 *   threshold - initial threshold
 *   z   - z array
 ************************************************************/

int java_mser3d_(name, nt, time, nx, ny, nz, threshold, grid) 
  char *name;
  int *nt;
  double time[];
  int nx[];
  int ny[];
  int nz[]; 
  double *threshold;
  double grid[];
{
   int sockfd;
   int ntsize = *nt;
   int name_length, index, i, j, k, size;
   double thre = *threshold;
   float thresh = (float)thre;
   int command = 1;

   sockfd = initialize2();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);

   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* write time and sizes and threshold */
   write_int4(sockfd, ntsize);     /* send no of time */
   write_float4(sockfd, thresh);

   /* write time data */
   index = 0;

   for (i=0; i<ntsize; i++) {
      write_float4(sockfd, (float)time[i]);
      write_int4(sockfd, nx[i]);
      write_int4(sockfd, ny[i]);
      write_int4(sockfd, nz[i]);

      size = nx[i] * ny[i] * nz[i];

      /* grid value */
      for (j=0; j<size; j++) {
         write_float4(sockfd, (float)grid[index]);
         index++;
      }   
   }
   
   close(sockfd);
   return 0;
}

int java_ser3d(char *name, double time, int nx, int ny, int nz, double threshold, double grid[]) {
   return java_ser3d_(name, &time, &nx, &ny, &nz, &threshold, grid);
}


int java_mser3d(char *name, int nt, double time[], int nx[], int ny[], int nz[], double threshold, double grid[]) {
   return java_mser3d_(name, &nt, time, nx, ny, nz, &threshold, grid);
}

