
import java.io.*;
import java.util.*;

/**
 * This file will generate the script file to run the Scivis.
 * To generate the automated script:
 *  1. Prepare the file "sv_input"
 *  2. Compile this program                
 *      % javac MakeScript.java
 *  3. Add the CLASSPATH environment
 *      % setenv CLASSPATH this_directory   (in UNIX)
 *      c> set CLASSPATH=this_directory     (in Windows95/NT)
 *  4. Run this program
 *     % java MakeScript
 *  5. Make the output file executable.
 *     % chmod +x your_ouput_file           (in UNIX)
 *     add extention ".bat"                 (in Window95/NT)
 *  6. Copy the output file which is defined in "sv_input" to 
 *     your working directory. 
 */

class MakeScript {
    public static void main(String[] args) {
        try {

           File tmpfile = new File("sv.tmp");
           PrintWriter writer = new PrintWriter(new FileWriter(tmpfile));
           
           FileInputStream propFile = new FileInputStream("sv_input");
           Properties p = new Properties(System.getProperties());
           p.load(propFile);

           String classpath = System.getProperty("java.class.path");
           String filesep = System.getProperty("file.separator");
           String pathsep = System.getProperty("path.separator");
           String filterpath = p.getProperty("filter_dir").trim();
           String libpath = p.getProperty("scivis_lib_dir").trim();
           String out_filename = p.getProperty("script_output").trim();
           String port_number = p.getProperty("scivis_port").trim();
           String configpath = p.getProperty("config_dir").trim() + filesep;

           writer.println("java.class.path=" + classpath + pathsep +
                      filterpath + filesep + pathsep +
                      libpath + filesep +"scivis.jar" + pathsep +
                      libpath + filesep +"swing.jar" + pathsep +
                      libpath + filesep +"motif.jar");
           writer.close();

           propFile = new FileInputStream("sv.tmp");
           p = new Properties(System.getProperties());
           p.load(propFile);
           System.setProperties(p);

           classpath = System.getProperty("java.class.path");
           tmpfile.delete();

           writer = new PrintWriter(new FileWriter(out_filename));
           writer.println("java -classpath " + classpath + " svserver -p " + 
                                     port_number + " -s " + configpath);
           writer.close();
        } catch (java.io.FileNotFoundException e) {
            System.err.println("Can't find \"sv_input\" file.");
        } catch (java.io.IOException e) {
            System.err.println("I/O failed.");
        } catch (java.lang.NullPointerException e) {
            System.err.println("Check if there is the missing field in \"sv_input\" file.");
        }

    }
}